// Java

import org.omg.CORBA.Any;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Policy;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.TCKind;
import org.omg.CosEventChannelAdmin.EventChannel;
import org.omg.CosEventChannelAdmin.EventChannelHelper;
import org.omg.CosEventChannelAdmin.ProxyPullSupplier;
import org.omg.CosEventChannelAdmin.ConsumerAdmin;
import org.omg.CosEventComm.Disconnected;
import org.omg.CosEventComm.PullSupplier;
import org.omg.CosEventComm.PullConsumerPOA;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;

public class PurePullConsumer extends PullConsumerPOA {

    private POA _poa;
    PullSupplier _pullSupplier;

    public PurePullConsumer(PullSupplier pullSupplier, POA poa) {

        _pullSupplier = pullSupplier;
        _poa = poa;
    }

    public void start() throws InterruptedException {

        Any anyVal = null;
        long longVal;
        short shortVal;
        double doubleVal;
        String stringVal;
        boolean hasEvent = false;
        BooleanHolder hasEventHolder = new BooleanHolder();

        System.out.println("Consumer in start of thread, sleeping...");
        Thread.currentThread().sleep(2000);
        while (true) {
            try {
                while (!hasEvent) {
                    Thread.currentThread().sleep(1000);
                    System.out.println("Consumer calling try_pull");
                    try {
                        anyVal = _pullSupplier.try_pull(hasEventHolder);
                        hasEvent = hasEventHolder.value;
                    } catch (SystemException se) {
                        System.err.println("Consumer caught CORBA " +
                                "SystemException: " + se);
                        return;
                    }
                }
                System.out.println("Consumer got event");
                TCKind kind = anyVal.type().kind();
                if (kind == TCKind.tk_long) {
                    longVal = anyVal.extract_long();
                    System.out.println("Consumer pulled long: " +
                            longVal);
                    if (longVal == 13) {
                        System.out.println("Consumer received #13, " +
                                "exiting...");
                        _pullSupplier.disconnect_pull_supplier();
                        return;
                    }
                } else if (kind == TCKind.tk_short) {
                    shortVal = anyVal.extract_short();
                    System.out.println("Consumer pulled short: " +
                            shortVal);
                } else if (kind == TCKind.tk_double) {
                    doubleVal = anyVal.extract_double();
                    System.out.println("Consumer pulled double: " +
                            doubleVal);
                } else if (kind == TCKind.tk_string) {
                    stringVal = anyVal.extract_string();
                    System.out.println("Consumer pulled string: " +
                            stringVal);
                 }
                 hasEvent = false;
            } catch (Disconnected d) {
                System.err.println("Consumer caught Disconnected " +
                        "Exception");
                return;
            }
        }
    }

    public void disconnect_pull_consumer() {

        System.out.println("PurePullConsumer." +
                "disconnect_pull_consumer()");

        try {
            byte[] objId = "PurePullConsumer".getBytes();
            _poa.deactivate_object(objId);
        } catch (Exception e) {
            System.err.println(e);
        }
    }

    public static void main(String[] args) {

        try {
            // initialize the ORB
            ORB orb = ORB.init(args, null);

            // 1. The client (Supplier or Consumer) first binds to the
            // Event Channel (which must be created by someone already,
            // perhaps the client itself).
            org.omg.CORBA.Object obj = orb.
                    resolve_initial_references("EventService");
            EventChannel eventChannel = EventChannelHelper.narrow(obj);
            if (eventChannel == null) {
                System.err.println("could not find EventChannel");
                return;
            }
            System.out.println("found the EventChannel");

            // 2. The client obtains an Admin object from the Event
            // Channel. A Consumer needs a ConsumerAdmin object, and a
            // Supplier needs a SupplierAdmin object.
            ConsumerAdmin consumerAdmin = eventChannel.for_consumers();

            // 3. The client obtains a proxy object from the Admin
            // object (a Consumer Proxy for a Supplier client, and a
            // Supplier Proxy for a Consumer client).
            ProxyPullSupplier proxySupplier = consumerAdmin.
                    obtain_pull_supplier();
            System.out.println("got a ProxyPullSupplier");

            // obtain a reference to the Root POA
            obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // create the policies for the push supplier POA
            Policy[] policies = new Policy[1];
            policies[0] = rootPOA.
                    create_lifespan_policy(LifespanPolicyValue.
                    PERSISTENT);

            // create the push supplier POA
            POAManager poaManager = rootPOA.the_POAManager();
            POA consumerPOA = rootPOA.create_POA("PullConsumerPOA",
                    poaManager, policies);

            // create and activate the push supplier
            PurePullConsumer pullConsumer = new
                    PurePullConsumer(proxySupplier, consumerPOA);
            String consumerName = "PurePullConsumer";
            byte[] objId = consumerName.getBytes();
            consumerPOA.activate_object_with_id(objId, pullConsumer);
            consumerPOA.the_POAManager().activate();
            consumerPOA.servant_to_reference(pullConsumer);

            // 4. The client adds the Supplier or Consumer to the Event
            // Channel via a connect() call.
            proxySupplier.connect_pull_consumer(pullConsumer._this());

            // 5. The client and/or the Event Channel transfer data via
            // the push(), pull(), and/or try_pull() calls.
            pullConsumer.start();

            System.out.println("Consumer is ending");
        } catch (Exception e) {
            System.err.println(e);
        }
    }
}