// Java

import org.omg.CORBA.Any;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Policy;
import org.omg.CORBA.SystemException;
import org.omg.CosEventChannelAdmin.EventChannel;
import org.omg.CosEventChannelAdmin.EventChannelHelper;
import org.omg.CosEventChannelAdmin.ProxyPushConsumer;
import org.omg.CosEventChannelAdmin.SupplierAdmin;
import org.omg.CosEventComm.Disconnected;
import org.omg.CosEventComm.PushConsumer;
import org.omg.CosEventComm.PushSupplierPOA;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;

public class PurePushSupplier extends PushSupplierPOA {

    private ORB _orb;
    private POA _poa;
    private PushConsumer _pushConsumer;

    public PurePushSupplier(PushConsumer pushConsumer, ORB orb, POA poa)
            {

        _pushConsumer = pushConsumer;
        _orb = orb;
        _poa = poa;
    }

    public void start() throws Disconnected, InterruptedException {

        Any any = _orb.create_any();

        // push some events to the channel

        any.insert_long(555555555);
        _pushConsumer.push(any);
        System.out.println("Supplier: just pushed a long value");
        Thread.currentThread().sleep(1000);

        any.insert_short((short)100);
        _pushConsumer.push(any);
        System.out.println("Supplier: just pushed a short value");

        any.insert_double(999999.999999);
        _pushConsumer.push(any);
        System.out.println("Supplier: just pushed a double value");

        any.insert_string("And the ubiquitous Hello World!");
        _pushConsumer.push(any);
        System.out.println("Supplier: just pushed a string");
        Thread.currentThread().sleep(5000);

        any.insert_long(13);
        _pushConsumer.push(any);

        _pushConsumer.disconnect_push_consumer();
    }

    public void disconnect_push_supplier() {

        System.out.println("PurePushSupplier." +
                "disconnect_push_supplier()");

        try {
            byte[] objId = "PurePushSupplier".getBytes();
            _poa.deactivate_object(objId);
        } catch (Exception e) {
            System.out.println(e);
        }
    }

    public static void main(String[] args) {

        try {
            // initialize the ORB
            ORB orb = ORB.init(args, null);

            // 1. The client (Supplier or Consumer) first binds to the
            // Event Channel (which must be created by someone already,
            // perhaps the client itself).
            org.omg.CORBA.Object obj = orb.
                    resolve_initial_references("EventService");
            EventChannel eventChannel = EventChannelHelper.narrow(obj);
            if (eventChannel == null) {
                System.err.println("could not find EventChannel");
                return;
            }
            System.out.println("found the EventChannel");

            // 2. The client obtains an Admin object from the Event
            // Channel. A Consumer needs a ConsumerAdmin object, and a
            // Supplier needs a SupplierAdmin object.
            SupplierAdmin supplierAdmin = eventChannel.for_suppliers();

            // 3. The client obtains a proxy object from the Admin
            // object (a Consumer Proxy for a Supplier client, and a
            // Supplier Proxy for a Consumer client).
            ProxyPushConsumer proxyConsumer = supplierAdmin.
                    obtain_push_consumer();
            System.out.println("got a ProxyPushConsumer");

            // obtain a reference to the Root POA
            obj = orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(obj);

            // create the policies for the push supplier POA
            Policy[] policies = new Policy[1];
            policies[0] = rootPOA.
                    create_lifespan_policy(LifespanPolicyValue.
                    PERSISTENT);

            // create the push supplier POA
            POAManager poaManager = rootPOA.the_POAManager();
            POA supplierPOA = rootPOA.create_POA("pushSupplierPOA",
                    poaManager, policies);

            // create and activate the push supplier
            PurePushSupplier  pushSupplier = new
                    PurePushSupplier(proxyConsumer, orb, supplierPOA);
            String supplierName = "PurePushSupplier";
            byte[] objId = supplierName.getBytes();
            supplierPOA.activate_object_with_id(objId, pushSupplier);
            supplierPOA.the_POAManager().activate();
            supplierPOA.servant_to_reference(pushSupplier);

            // 4. The client adds the Supplier or Consumer to the Event
            // Channel via a connect() call.
            proxyConsumer.connect_push_supplier(pushSupplier._this());

            // 5. The client and/or the Event Channel transfer data via
            // the push(), pull(), and/or try_pull() calls.
            pushSupplier.start();

            System.out.println("Supplier is ending");
        } catch (Exception e) {
            System.err.println(e);
        }
    }
}