
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <sstream>

#include <omg/CosNaming.hh>
#include "BookRepository.hh"


CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// read_reference() -- read an object reference from file.
static 
CORBA::Object_ptr
read_reference(
    const char*      file
)
{
    ifstream ifs(file);
    CORBA::String_var strV;
    ifs >> strV;
    if (!ifs) {
        cerr << "Error reading object reference from " << file << endl;
        return CORBA::Object::_nil();
    }
    return global_orb->string_to_object(strV);
}


int
main(int argc, char **argv)
{
    try
    {
        CORBA::Object_var objV;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

        // Parse command line arguments:
        if (argc != 4) {
            cout << "Usage: bk_borrow "
                 << "<your_first_name> <your_second_name> <book_id>" << endl;
            exit(0);
        }
        const char * borrower_first_name  = argv[1];
        const char * borrower_second_name = argv[2];
        const char * isbn_string          = argv[3];

        // Obtain Naming Service reference.
        CosNaming::NamingContext_var rootContextV;
        objV = global_orb->resolve_initial_references("NameService");
        rootContextV = CosNaming::NamingContext::_narrow(objV);
        if (CORBA::is_nil(rootContextV) ) {
            cerr << "error: failed to narrow root naming context." << endl;
            exit(1);
        }

        // Obtain 'BookRepository::BorrowableCollection' object reference.
        CosNaming::Name objectName(2);
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("Borrowable");
        objectName[1].kind = CORBA::string_dup("");
        objV = rootContextV->resolve(objectName);
        BookRepository::BorrowableCollection_var borrowableCollectionV =
            BookRepository::BorrowableCollection::_narrow(objV);
        if (CORBA::is_nil(borrowableCollectionV )) 
        {
            cerr << "error: failed to narrow BorrowableCollection." << endl;
            exit(1);
        }
 
        // Invoke 'BookRepository::BorrowableCollection' object.
        CORBA::Long book_id;
        std::string s = isbn_string;
        std::istringstream ist(s);
        ist >> book_id;
        BookRepository::PersonName borrower;
        borrower.first_name  = CORBA::string_dup(borrower_first_name);
        borrower.second_name = CORBA::string_dup(borrower_second_name);
        BookRepository::Date_var return_dateV;
        try {
            borrowableCollectionV->borrow_book(
                                       book_id,
                                       borrower,
                                       return_dateV.out()
                                   );
            cout << "Book [ISBN=" << book_id << "] has been borrowed." << endl;
            cout << "Please return by: "
                 << return_dateV->day   << "/"
                 << return_dateV->month << "/"
                 << return_dateV->year  << endl;
        }
        catch (BookRepository::BorrowableCollection::Unavailable& bk_ex) {
            cout << "Sorry, book [ISBN=" << book_id
                 << "] is unavailable until "
                 << bk_ex.when_available.day   << "/"
                 << bk_ex.when_available.month << "/"
                 << bk_ex.when_available.year  << endl;
        }
    }
    catch(CORBA::Exception &ex) 
    {
        cerr << "Unexpected CORBA exception: " << ex << endl;
    }

    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return 0;
}


