
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <sstream>
#include <omg/CosNaming.hh>
#include "BookRepository.hh"


CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// read_reference() -- read an object reference from file.
static 
CORBA::Object_ptr
read_reference(
    const char*      file
)
{
    ifstream ifs(file);
    CORBA::String_var strV;
    ifs >> strV;
    if (!ifs) {
        cerr << "Error reading object reference from " << file << endl;
        return CORBA::Object::_nil();
    }
    return global_orb->string_to_object(strV);
}


int
main(int argc, char **argv)
{
    try
    {
        CORBA::Object_var objV;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

        // Parse command line arguments:
        if (argc != 1) {
            cout << "Usage: bk_collection " << endl;
            exit(0);
        }

        // Obtain Naming Service reference.
        CosNaming::NamingContext_var rootContextV;
        objV = global_orb->resolve_initial_references("NameService");
        rootContextV = CosNaming::NamingContext::_narrow(objV);
        if (CORBA::is_nil(rootContextV.in()) ) {
            cerr << "error: failed to narrow root naming context." << endl;
            exit(1);
        }

        // Obtain 'BookRepository::Collection' object reference.
        CosNaming::Name objectName(2);
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("PlainCollection");
        objectName[1].kind = CORBA::string_dup("");
        objV = rootContextV->resolve(objectName);
        BookRepository::Collection_var collectionV =
            BookRepository::Collection::_narrow(objV);
        if (CORBA::is_nil(collectionV.in() )) 
        {
            cerr << "error: failed to narrow to Collection." << endl;
            exit(1);
        }
 
        // Invoke 'BookRepository::Collection' object.
        cout << "Details of book collection:" << endl;
        CORBA::String_var name_strV;
        name_strV = collectionV->name_of_collection();
        cout << "\tName of collection = \"" << name_strV.in() << "\"" << endl;
        CORBA::Long n_books;
        n_books = collectionV->number_of_books();
        cout << "\tNumber of books = " << n_books << endl;

        // Try changing the name of the book collection.
        cout << "Changing name of book collection..." << endl;
        collectionV->name_of_collection("Brand new collection!");
        cout << "checking name of collection..." << endl;
        name_strV = collectionV->name_of_collection();
        cout << "\tName of collection = \"" << name_strV.in() << "\"" << endl;
    }
    catch(CORBA::Exception &ex) 
    {
        cerr << "Unexpected CORBA exception: " << ex << endl;
    }

    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return 0;
}


