
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>

#include <omg/CosNaming.hh>
#include "BookRepository.hh"


CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// read_reference() -- read an object reference from file.
static 
CORBA::Object_ptr
read_reference(
    const char*      file
)
{
    ifstream ifs(file);
    CORBA::String_var strV;
    ifs >> strV;
    if (!ifs) {
        cerr << "Error reading object reference from " << file << endl;
        return CORBA::Object::_nil();
    }
    return global_orb->string_to_object(strV);
}


int
main(int argc, char **argv)
{
    try
    {
        CORBA::Object_var objV;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

        // Parse command line arguments:
        if (argc != 2) {
            cout << "Usage: bk_search " << " <book_title>" << endl;
            exit(0);
        }
        const char * book_title = argv[1];

        // Obtain Naming Service reference.
        CosNaming::NamingContext_var rootContextV;
        objV = global_orb->resolve_initial_references("NameService");
        rootContextV = CosNaming::NamingContext::_narrow(objV);
        if (CORBA::is_nil(rootContextV.in()) ) {
            cerr << "error: failed to narrow root naming context." << endl;
            exit(1);
        }

        // Obtain 'BookRepository::SearchableCollection' object reference.
        CosNaming::Name objectName(2);
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("Searchable");
        objectName[1].kind = CORBA::string_dup("");
        objV = rootContextV->resolve(objectName);
        BookRepository::SearchableCollection_var searchableCollectionV =
            BookRepository::SearchableCollection::_narrow(objV);
        if (CORBA::is_nil(searchableCollectionV.in() )) 
        {
            cerr << "error: failed to narrow to SearchableCollection." << endl;
            exit(1);
        }
        
        // Invoke on BookRepository::SearchableCollection object.
        BookRepository::BookDetailsSeq_var detailsSeqV;
        CORBA::Boolean wasFound;
        wasFound = searchableCollectionV->find_by_title(
                                      book_title,
                                      detailsSeqV.out()
                                  );
        if (wasFound) {
            cout << "Found the following matching titles:" << endl;
        }
        else {
            cout << "No matching titles." << endl;
        }

        // Print out the list of books (if any)
        for (CORBA::ULong i=0; i < detailsSeqV->length(); i++) {
            cout << "Book[" << i << "]" << endl;
            cout << "\tAuthor: " << detailsSeqV[i].author.second_name
                 << ", "         << detailsSeqV[i].author.first_name << endl;
            cout << "\tTitle:  " << detailsSeqV[i].title << endl;
            cout << "\tISBN:   " << detailsSeqV[i].book_id << endl;
            cout << "\tPublication Date: "
                 << detailsSeqV[i].publication_date.day   << "/"
                 << detailsSeqV[i].publication_date.month << "/"
                 << detailsSeqV[i].publication_date.year  << endl;
        }
    }
    catch(CORBA::Exception &ex) 
    {
        cerr << "Unexpected CORBA exception: " << ex << endl;
    }

    // Ensure that the ORB is properly shutdown and cleaned up.
    //
    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return 0;
}


