package Pure.BookRepository;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import java.io.*;

import java.text.DateFormat;


public class server {

   public static ORB global_orb = null;



   // write_reference() -- export object reference to file.
   static void write_reference(
      org.omg.CORBA.Object            ref,
      String                          objref_file
   )
   {
      String stringified_ref = global_orb.object_to_string(ref);
      System.out.println(
          "Writing stringified object reference to " + objref_file
      );
    
      try
      {
         FileWriter store = new FileWriter(objref_file);
         store.write(stringified_ref);
         store.flush();
         store.close();
      }
      catch (IOException ex)
      {
         System.out.println("Failed to write to " + objref_file);
      }
   }

   // create_simple_poa() -- Create a POA for simple servant management.
   static POA create_simple_poa(
      String           poa_name,
      POA              parent_poa,
      POAManager       poa_manager
   )
   {
      // Create a policy list. Policies not set in the list get default values.
      org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[1];
      int i = 0;
      POA new_poa = null;

      // Make the POA single threaded.
      policies[i++] = parent_poa.create_thread_policy(
                          ThreadPolicyValue.SINGLE_THREAD_MODEL
                      );
      if(! (i==1) ) 
      {
         System.out.println("Policy creation failed");
         System.exit(1);
      }

      try
      {
         new_poa = parent_poa.create_POA(poa_name, poa_manager, policies);
      }
      catch (org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
      {
         System.out.println(
             "Failed to create the POA with exception : " + ex.toString()
         );
         System.exit(1);
      }
      catch (org.omg.PortableServer.POAPackage.InvalidPolicy ex)
      {
         System.out.println(
             "Failed to create the POA with exception : " + ex.toString()
         );
         System.exit(1);
      }
    
      return new_poa;

   }


    public static void main(String args[])
    {
        // Variables to hold the servants.
        Servant the_BookRepository_Collection = null;
        Servant the_BookRepository_SearchableCollection = null;
        Servant the_BookRepository_BorrowableCollection = null;

        try
        {
            org.omg.CORBA.Object obj = null;

            // Initialise the ORB and Root POA.
            //
            System.out.println( "Initializing the ORB" );
            try
            {
                global_orb = ORB.init(args, null);
                obj = global_orb.resolve_initial_references("RootPOA");
            }
            catch (org.omg.CORBA.ORBPackage.InvalidName ex)
            {
                System.out.println(
                    "Caught an unexpected exception while resolving"
                    + "to the RootPOA : " + ex.toString()
                );
                System.exit(1);
            }
            POA root_poa = POAHelper.narrow(obj);
            POAManager root_poa_manager = root_poa.the_POAManager();

            // Now create a simple POA.
            org.omg.PortableServer.POA my_poa
                = create_simple_poa("my_poa", root_poa, root_poa_manager);

            // Obtain Naming Service reference.
            org.omg.CosNaming.NamingContext rootContext = null;
            obj = global_orb.resolve_initial_references("NameService");
            rootContext = org.omg.CosNaming.NamingContextHelper.narrow(obj);

            // Make sure that a 'BookRepository' naming context exists.
            org.omg.CosNaming.NameComponent[] objectName
                = new org.omg.CosNaming.NameComponent[1];
            objectName[0]
                = new org.omg.CosNaming.NameComponent("BookRepository", "");
            try {
                rootContext.bind_new_context(objectName);
            }
            catch (Exception e) {  }


            // Create servants and export object references.
            byte [] oid;

            try{
                // Create a servant for Collection.
                the_BookRepository_Collection =
                   Pure.BookRepository.CollectionImpl._create(my_poa);
                oid = my_poa.activate_object(the_BookRepository_Collection);
                obj = my_poa.id_to_reference(oid);
                write_reference(obj,"BookRepository_Collection.ref");
                objectName = new org.omg.CosNaming.NameComponent[2];
                objectName[0]
                    = new org.omg.CosNaming.NameComponent("BookRepository", "");
                objectName[1]
                    = new org.omg.CosNaming.NameComponent("PlainCollection", "");
                rootContext.rebind(objectName, obj);

                // Create a servant for SearchableCollection.
                the_BookRepository_SearchableCollection =
                   Pure.BookRepository.SearchableCollectionImpl._create(my_poa);
                oid = my_poa.activate_object(
                          the_BookRepository_SearchableCollection
                      );
                obj = my_poa.id_to_reference(oid);
                write_reference(obj,"BookRepository_SearchableCollection.ref");
                objectName[0]
                    = new org.omg.CosNaming.NameComponent("BookRepository", "");
                objectName[1]
                    = new org.omg.CosNaming.NameComponent("Searchable", "");
                rootContext.rebind(objectName, obj);

                // Create a servant for Pure.BookRepository.BorrowableCollection.
                the_BookRepository_BorrowableCollection =
                    Pure.BookRepository.BorrowableCollectionImpl._create(my_poa);
                oid = my_poa.activate_object(
                          the_BookRepository_BorrowableCollection
                      );
                obj = my_poa.id_to_reference(oid);
                write_reference(obj,"BookRepository_BorrowableCollection.ref");
                objectName[0]
                    = new org.omg.CosNaming.NameComponent("BookRepository", "");
                objectName[1]
                    = new org.omg.CosNaming.NameComponent("Borrowable", "");
                rootContext.rebind(objectName, obj);
            }
            catch (Exception ex)
            {
                System.out.println(ex);
                ex.printStackTrace();
                System.exit(1);
            }

            // Activate the POA Manager.
            try
            {
                root_poa_manager.activate();
            }
            catch (org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
            {
                System.out.println( "error: could activate POA manager" + ex);
                System.exit(1);
            }

            //Let the ORB process requests.
            System.out.println("Waiting for requests..." );
            global_orb.run();
        }
        catch (Exception ex)
        {
            System.out.println("error: unexpected exception: " + ex);
        }

        try 
        {
            global_orb.shutdown(true);
            //global_orb.destroy();
        }
        catch (Exception e) 
        { 
            // Do nothing.
        }        
        return;
    }
}
