//----------------------------------------
// Includes
//----------------------------------------


#include <iostream.h>
#include <fstream.h>
#include <omg/orb.hh>
#include <omg/PortableServer.hh>
#include <omg/CosNaming.hh>

#include "NameUtil.h"

//----------------------------------------
// Implementation of class: NameUtil
//----------------------------------------

//----------------------------------------
// C'tor and D'tor
//----------------------------------------
NameUtil::NameUtil() {}
NameUtil::~NameUtil() {}

//----------------------------------------
// method: 'printName()'
//
// purpose: Print 'name' to 'cout' in standard stringified
//          format.
//----------------------------------------
void
NameUtil::printName(const CosNaming::Name& name)
{
     for (CORBA::ULong i=0; i<name.length(); i++) {
          if (strlen( (const char*) name[i].id)==0) {
	       //Special case: zero length 'id'
	       cout << "." << name[i].kind;
	  }
	  else {
	       //Regular case
	       cout << name[i].id;
               if (strlen( (const char*) name[i].kind)!=0) {
	            cout << "." << name[i].kind;
	       }
	  }
	  if (i < name.length()-1) {
	       cout << "/";
	  }
     }
}

//----------------------------------------
// method: 'createContextPath()'
//
// purpose: For each 'NameComponent' in 'name',
//          create a corresponding 'NamingContext'.
//----------------------------------------
void
NameUtil::createContextPath(
          const CosNaming::NamingContext_ptr nc,
          const CosNaming::Name&  name
	  )
{
    int isNotFound = 0;
    CORBA::ULong lengthMissing = 0;

    CosNaming::NamingContext_var tmpCtxVar;
    try {
        tmpCtxVar = nc->bind_new_context(name);
    }
    catch (CosNaming::NamingContext::NotFound& nf) {
        isNotFound = 1;
	lengthMissing = nf.rest_of_name.length();
    }
    if (lengthMissing==name.length() ) {
        cerr << "This cannot happen!" << endl;
        lengthMissing--;
    }

    if (isNotFound) {
        for (CORBA::ULong l=name.length()-lengthMissing;
	                  l <= name.length();
			  l++)
        {
             CosNaming::Name tmpName = name;
             tmpName.length(l);
             tmpCtxVar = nc->bind_new_context(tmpName);
	}
    }

}

//----------------------------------------
// method: 'bindObjectPath()'
//
// purpose: For the first [0, length-2] NameComponents of
//          'name', create a corresponding 'NamingContext'.
//          For the length-1 NameComponent of 'name', bind it
//          to the object reference 'obj'.
//----------------------------------------
void
NameUtil::bindObjectPath(
          const CosNaming::NamingContext_ptr nc,
          const CosNaming::Name&  name,
	  const CORBA::Object_ptr obj
	  )
{
     try {
          nc->rebind(name, obj);
     }
     catch (CosNaming::NamingContext::NotFound& ) {
         CosNaming::Name tmpName = name;
         tmpName.length(tmpName.length()-1);
         createContextPath(nc, tmpName);

         nc->bind(name, obj);
     }
}

//----------------------------------------
// method: 'listBindings()'
//
// purpose: 
//----------------------------------------
CosNaming::BindingList * 
NameUtil::listBindings(
          const CosNaming::NamingContext_ptr nc,
          const CosNaming::Name&  name,
	  CORBA::ULong max_list_size
	  )
{
     CosNaming::BindingList_var     basicListV;
     CosNaming::BindingIterator_var bIterV;

     CORBA::Object_var objV;
     CosNaming::NamingContext_var tmpContextV;

     if (name.length()==0) {
          tmpContextV = CosNaming::NamingContext::_duplicate(nc);
     }
     else {
          objV = nc->resolve(name);
          tmpContextV = CosNaming::NamingContext::_narrow(objV);
     }
     if (CORBA::is_nil(tmpContextV)) {
          cerr << "listBindings: Nil context" << endl;
	  return 0;
     }

     tmpContextV->list(max_list_size, basicListV.out(), bIterV.out());

     CORBA::Long     max_remaining = max_list_size - basicListV->length();
     CORBA::Boolean  moreBindings  = !CORBA::is_nil(bIterV);

     if (moreBindings) {
	  while (moreBindings && (max_remaining > 0) ) {
               CosNaming::BindingList_var tmpListV;

	       moreBindings = bIterV->next_n(max_remaining, tmpListV.out());
               
	       //Append 'tmpListV' to 'basicListV'
	       CORBA::ULong basicListLen = basicListV->length();
	       basicListV->length(basicListLen+tmpListV->length());
	       for (CORBA::ULong i=0; i < tmpListV->length(); i++) {
	            (*basicListV)[i+basicListLen] = (*tmpListV)[i];
	       }

	       //Re-calculate 'max_remaining'
               max_remaining = max_list_size - basicListV->length();
	  }
	  bIterV->destroy();
     }

     return basicListV._retn();
}

//----------------------------------------
// method: 'printBindings()'
//
// purpose: 
//----------------------------------------
void
NameUtil::printBindings(
          const CosNaming::NamingContext_ptr nc,
          const CosNaming::Name&  name
	  )
{
     CosNaming::BindingList_var blV;

     blV = NameUtil::listBindings(nc,
                        name,
			10000   // 'max_list_size'
			);

     for (CORBA::ULong i=0; i<blV->length(); i++)
     {
          NameUtil::printName( (*blV)[i].binding_name );
	  cout << "\t";
	  switch ( (*blV)[i].binding_type ) {
	     case CosNaming::nobject:
	        cout << "Object" << endl;
		break;
	     case CosNaming::ncontext:
	        cout << "Context" << endl;
		break;
	     default:
	        cout << "?????" << endl;
	  }
     }
}

//----------------------------------------
// method: 'recursiveUnbind()'
//
// purpose: 
//----------------------------------------
void
NameUtil::recursiveUnbind(
          const CosNaming::NamingContext_ptr nc,
          const CosNaming::Name&  name
	  )
{
     CORBA::Object_var objV;
     CosNaming::NamingContext_var tmpContextV;

     objV = nc->resolve(name);
     tmpContextV = CosNaming::NamingContext::_narrow(objV);
     if (CORBA::is_nil(tmpContextV)) {
          cerr << "recursiveUnbind: Nil context reference" << endl;
	  return;
     }

     CosNaming::BindingList_var blV;
     CosNaming::Name tmpName;
     tmpName.length(0);

     blV = NameUtil::listBindings(tmpContextV.in(),
                        tmpName,
			10000   // 'max_list_size'
			);

     for (CORBA::ULong i=0; i<blV->length(); i++)
     {
	  tmpName = (*blV)[i].binding_name;

	  if      ( (*blV)[i].binding_type==CosNaming::nobject)
	  {
	       tmpContextV->unbind(tmpName);
	  }
	  else if ( (*blV)[i].binding_type==CosNaming::ncontext)
	  {
	       NameUtil::recursiveUnbind(tmpContextV.in(), tmpName);
	  }
     }
     nc->unbind(name);
     tmpContextV->destroy();
}
