#ifndef _DBBASE_H_
#define _DBBASE_H_

#include "RecycleBroker.hh"

//---------------------
// An Abstract Class
//---------------------

class DBBase;

// Global instance of the 'DBBase' class
extern DBBase* global_db;

class DBBase {

public:
    // Return value is 'customer_id'
    // A return value of '-1' indicates function failure.
    virtual RecycleBroker::KeyType
    create_customer_details(
        const RecycleBroker::CustomerDetails& initialData,
	RecycleBroker::KeyType                branch_id
    )  =0;
    
    // Caller is responsible for deleting returned 'dataP' parameter.
    virtual CORBA::Boolean
    find_customer_details(
	RecycleBroker::KeyType                customer_id,
        RecycleBroker::CustomerDetailsFull*&  dataP
    )  =0;

    virtual RecycleBroker::CustomerIdSeq*
    find_customer_details_by_name(
	const char*               name
    )  =0;

    virtual CORBA::Boolean
    update_customer_details(
	RecycleBroker::KeyType                customer_id,
        const RecycleBroker::CustomerDetails& data
    )  =0;
    
    // Return value is 'wasteitem_id'
    // A return value of '-1' indicates function failure.
    virtual RecycleBroker::KeyType
    create_wasteitem_details(
        const RecycleBroker::WasteItemDetails& initialData,
	RecycleBroker::KeyType                 branch_id,
	RecycleBroker::KeyType                 customer_id
    )  =0;
    
    // Caller is responsible for deleting returned 'dataP' parameter.
    virtual CORBA::Boolean
    find_wasteitem_details(
	RecycleBroker::KeyType                 wasteitem_id,
        RecycleBroker::WasteItemDetailsFull*&  dataP
    )  =0;

    virtual CORBA::Boolean
    find_wasteitem_details(
        const RecycleBroker::WasteItemIdSeq&   id_seq,
        RecycleBroker::WasteItemDetailsSeq*&   dataP
    )  =0;

    virtual CORBA::Boolean
    find_wasteitem_by_waste(
        RecycleBroker::WasteType               waste,
        RecycleBroker::WasteItemIdSeq*&        id_seqP
    )  =0;

    virtual CORBA::Boolean
    find_wasteitem_by_branch(
        RecycleBroker::KeyType                 branch_id,
        RecycleBroker::WasteItemIdSeq*&        id_seqP
    )  =0;

    virtual CORBA::Boolean
    find_all_wasteitem(
        RecycleBroker::WasteItemIdSeq*&        id_seqP
    )  =0;

    virtual CORBA::Boolean
    update_wasteitem_details(
	RecycleBroker::KeyType                 wasteitem_id,
        const RecycleBroker::WasteItemDetails& data
    )  =0;
    
    // Destructor
    virtual ~DBBase();

protected:
    // Protected functions.
    DBBase();

private:
    //Not defined in this class
    DBBase(const DBBase &);
    DBBase & operator=(const DBBase &);
};

#endif
