// Class: RecycleBroker_CustomerAdmin_i
// A POA servant which implements of the RecycleBroker::CustomerAdmin interface
//

#ifndef RECYCLEBROKER_CUSTOMERADMIN_I_H_
#define RECYCLEBROKER_CUSTOMERADMIN_I_H_

#include "RecycleBrokerS.hh"


class RecycleBroker_CustomerAdmin_i :
    public virtual PortableServer::RefCountServantBase,
    public virtual POA_RecycleBroker::CustomerAdmin
{
public:
    RecycleBroker_CustomerAdmin_i(
        CORBA::Boolean          hasAdminPrivileges,
        RecycleBroker::KeyType  branch_id
    );

    virtual ~RecycleBroker_CustomerAdmin_i();

    // _create() -- create a new servant.
    // Hides the difference between direct inheritance and tie servants.
    //    
    static POA_RecycleBroker::CustomerAdmin*
    _create(
        CORBA::Boolean          hasAdminPrivileges,
        RecycleBroker::KeyType  branch_id
    );


    // IDL operations
    //
    virtual RecycleBroker::Customer_ptr
    create(
        const RecycleBroker::CustomerDetails& initialData,
        RecycleBroker::KeyType_out      customer_id
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NoPermission
    ));

    virtual RecycleBroker::Customer_ptr
    find(
        RecycleBroker::KeyType          customer_id
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NoPermission,
        RecycleBroker::NotFound
    ));

    virtual RecycleBroker::CustomerIdSeq*
    find_by_name(
        const char*                     name
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NoPermission,
        RecycleBroker::NotFound
    ));

    virtual PortableServer::POA_ptr
    _default_POA();

private: 
    // Private members
    CORBA::Boolean               m_hasAdminPrivileges;
    RecycleBroker::KeyType       m_branch_id;

    // The following are not implemented
    RecycleBroker_CustomerAdmin_i(const RecycleBroker_CustomerAdmin_i &);
    RecycleBroker_CustomerAdmin_i& operator=(const RecycleBroker_CustomerAdmin_i &);
}; 

#endif
