// Servant which implements the RecycleBroker::Office interface.
//

#include <stdlib.h>
#include <iostream.h>
#include "RecycleBroker_Office_i.h"
#include "RecycleBroker_Browsing_i.h"
#include "RecycleBroker_Selling_i.h"
#include "RecycleBroker_Buying_i.h"
#include "RecycleBroker_CustomerAdmin_i.h"
#include "RecycleBroker_WasteItemAdmin_i.h"
#include "POAMapper.h"


// _create() -- create a new servant.
// Hides the difference between direct inheritance and tie servants
// For direct inheritance, simple create and return an instance of the servant.
// For tie, creates an instance of the tied class and the tie, return the tie.
//
POA_RecycleBroker::Office*
RecycleBroker_Office_i::_create(const char* address, CORBA::Long branch_id)
{
    return new RecycleBroker_Office_i(address, branch_id);
}


// RecycleBroker_Office_i constructor
RecycleBroker_Office_i::RecycleBroker_Office_i(
    const char* address,
    CORBA::Long branch_id
)
{
    // Initialise instance variables used for attributes
    m_address = address;
    m_branch_id = branch_id;
}


// ~RecycleBroker_Office_i destructor.
RecycleBroker_Office_i::~RecycleBroker_Office_i()
{
}


// get_browsing() -- Implements IDL operation "RecycleBroker::Office::get_browsing".
//
RecycleBroker::Browsing_ptr
RecycleBroker_Office_i::get_browsing() IT_THROW_DECL((
    CORBA::SystemException
))
{
    cout << "RecycleBroker_Office_i::get_browsing(): called." << endl;

    PortableServer::ServantBase_var the_BrowsingV;
    PortableServer::POA_var      child_poaV;
    PortableServer::ObjectId_var oidV;
    CORBA::Object_var            objV;
    RecycleBroker::Browsing_var  refV;

    the_BrowsingV = RecycleBroker_Browsing_i::_create(m_branch_id);
    child_poaV = the_BrowsingV->_default_POA();
    oidV = child_poaV->activate_object(the_BrowsingV.in() );
    objV = child_poaV->id_to_reference(oidV);
    refV = RecycleBroker::Browsing::_narrow(objV.in() );

    return refV._retn();
}


// get_selling() -- Implements IDL operation "RecycleBroker::Office::get_selling".
//
RecycleBroker::Selling_ptr
RecycleBroker_Office_i::get_selling() IT_THROW_DECL((
    CORBA::SystemException
))
{
    cout << "RecycleBroker_Office_i::get_selling(): called." << endl;

    PortableServer::ServantBase_var the_SellingV;
    PortableServer::POA_var      child_poaV;
    PortableServer::ObjectId_var oidV;
    CORBA::Object_var            objV;
    RecycleBroker::Selling_var   refV;

    the_SellingV = RecycleBroker_Selling_i::_create(m_branch_id);
    child_poaV = the_SellingV->_default_POA();
    oidV = child_poaV->activate_object(the_SellingV.in() );
    objV = child_poaV->id_to_reference(oidV);
    refV = RecycleBroker::Selling::_narrow(objV.in() );

    return refV._retn();
}


// get_buying() -- Implements IDL operation "RecycleBroker::Office::get_buying".
//
RecycleBroker::Buying_ptr
RecycleBroker_Office_i::get_buying() IT_THROW_DECL((
    CORBA::SystemException
))
{
    cout << "RecycleBroker_Office_i::get_buying(): called." << endl;

    PortableServer::ServantBase_var the_BuyingV;
    PortableServer::POA_var      child_poaV;
    PortableServer::ObjectId_var oidV;
    CORBA::Object_var            objV;
    RecycleBroker::Buying_var    refV;

    the_BuyingV = RecycleBroker_Buying_i::_create(m_branch_id);
    child_poaV = the_BuyingV->_default_POA();
    oidV = child_poaV->activate_object(the_BuyingV.in() );
    objV = child_poaV->id_to_reference(oidV);
    refV = RecycleBroker::Buying::_narrow(objV.in() );

    return refV._retn();
}


// get_customer_admin() -- Implements IDL operation "RecycleBroker::Office::get_customer_admin".
//
RecycleBroker::CustomerAdmin_ptr
RecycleBroker_Office_i::get_customer_admin() IT_THROW_DECL((
    CORBA::SystemException
))
{
    cout << "RecycleBroker_Office_i::get_customer_admin(): called." << endl;

    PortableServer::ServantBase_var the_CustomerAdminV;
    PortableServer::POA_var      child_poaV;
    PortableServer::ObjectId_var oidV;
    CORBA::Object_var            objV;
    RecycleBroker::CustomerAdmin_var    refV;

    the_CustomerAdminV
        = RecycleBroker_CustomerAdmin_i::_create(
              1,             // hasAdminPrivileges
              m_branch_id    // branch_id
          );
    child_poaV = the_CustomerAdminV->_default_POA();
    oidV = child_poaV->activate_object(the_CustomerAdminV.in() );
    objV = child_poaV->id_to_reference(oidV);
    refV = RecycleBroker::CustomerAdmin::_narrow(objV.in() );

    return refV._retn();
}


// get_waste_item_admin() -- Implements IDL operation "RecycleBroker::Office::get_waste_item_admin".
//
RecycleBroker::WasteItemAdmin_ptr
RecycleBroker_Office_i::get_waste_item_admin() IT_THROW_DECL((
    CORBA::SystemException
))
{
    cout << "RecycleBroker_Office_i::get_waste_item_admin(): called." << endl;

    PortableServer::ServantBase_var the_WasteItemAdminV;
    PortableServer::POA_var      child_poaV;
    PortableServer::ObjectId_var oidV;
    CORBA::Object_var            objV;
    RecycleBroker::WasteItemAdmin_var    refV;

    the_WasteItemAdminV
        = RecycleBroker_WasteItemAdmin_i::_create(
              1,             // hasAdminPrivileges
              m_branch_id,   // branch_id
              0              // customer_id = 0  => Administrator
          );
    child_poaV = the_WasteItemAdminV->_default_POA();
    oidV = child_poaV->activate_object(the_WasteItemAdminV.in() );
    objV = child_poaV->id_to_reference(oidV);
    refV = RecycleBroker::WasteItemAdmin::_narrow(objV.in() );

    return refV._retn();
}


// address() -- Accessor for IDL attribute "RecycleBroker::Office::address".
//
char*
RecycleBroker_Office_i::address() IT_THROW_DECL((CORBA::SystemException))

{
    return CORBA::string_dup(m_address);
}


// branch_id() -- Accessor for IDL attribute "RecycleBroker::Office::branch_id".
//
CORBA::Long
RecycleBroker_Office_i::branch_id() IT_THROW_DECL((CORBA::SystemException))

{
    return m_branch_id;
}

PortableServer::POA_ptr
RecycleBroker_Office_i::_default_POA()
{
    CORBA::String_var rep_idV = RecycleBroker::_tc_Office->id();
    return POAMapper::getPOA(rep_idV.in() );
}

