// Servant which implements the RecycleBroker::Selling interface.
//

#include <stdlib.h>
#include <iostream.h>
#include <string>
#include "RecycleBroker_Selling_i.h"
#include "RecycleBroker_CustomerAdmin_i.h"
#include "RecycleBroker_WasteItemAdmin_i.h"
#include "POAMapper.h"


// _create() -- create a new servant.
POA_RecycleBroker::Selling*
RecycleBroker_Selling_i::_create(RecycleBroker::KeyType branch_id)
{
    return new RecycleBroker_Selling_i(branch_id);
}


// RecycleBroker_Selling_i constructor
RecycleBroker_Selling_i::RecycleBroker_Selling_i(
    RecycleBroker::KeyType branch_id
) :
    m_current_status(RecycleBroker::Selling::INITIAL),
    m_branch_id(branch_id),
    m_customer_id(0),
    m_customer_admin_iP(0),
    m_wasteitem_admin_iP(0)
{
}


// ~RecycleBroker_Selling_i destructor.
//
RecycleBroker_Selling_i::~RecycleBroker_Selling_i()
{
    delete m_customer_admin_iP;
    delete m_wasteitem_admin_iP;
}


// create_customer() -- Implements IDL operation "RecycleBroker::Selling::create_customer".
//
RecycleBroker::Customer_ptr
RecycleBroker_Selling_i::create_customer(
    const RecycleBroker::CustomerDetails& initialData,
    RecycleBroker::KeyType_out      customer_id
) IT_THROW_DECL((
    CORBA::SystemException
))
{
    cout << "RecycleBroker_Selling_i::create_customer(): called." << endl;

    if (m_current_status != RecycleBroker::Selling::INITIAL) {
        cerr << "error: cannot log on twice in the same session." << endl;
        return RecycleBroker::Customer::_nil();
    }

    // Create a new customer record
    if (!m_customer_admin_iP) { _init_customer_admin(); }
    RecycleBroker::Customer_var refV;
    refV = m_customer_admin_iP->create(initialData, customer_id);

    // Log on
    m_customer_id = customer_id;
    m_current_status = RecycleBroker::Selling::LOGGED_ON;

    return refV._retn();
}


// log_on() -- Implements IDL operation "RecycleBroker::Selling::log_on".
RecycleBroker::Customer_ptr
RecycleBroker_Selling_i::log_on(
    const char*                     name,
    const char*                     password
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NoPermission
))
{
    cout << "RecycleBroker_Selling_i::log_on(): called." << endl;

    if (m_current_status != RecycleBroker::Selling::INITIAL) {
        cerr << "error: cannot log on twice in the same session." << endl;
        return RecycleBroker::Customer::_nil();
    }

    // Find customer record
    if (!m_customer_admin_iP) { _init_customer_admin(); }
    RecycleBroker::CustomerIdSeq_var seqV;
    seqV = m_customer_admin_iP->find_by_name(name);
    if (seqV->length() > 1) {
        cerr << "error: duplicate records exist for " << name << endl;
        throw RecycleBroker::NoPermission();
    }
    else if (seqV->length() != 1) {
        throw RecycleBroker::NoPermission();
    }

    // Found unique 'customer_id'
    RecycleBroker::KeyType customer_id    = seqV[0];

    // Get obj ref for 'Customer' object
    RecycleBroker::Customer_var refV;
    refV = m_customer_admin_iP->find(customer_id);

    // Check 'password'
    CORBA::String_var sV = refV->password();
    std::string s1(password);
    std::string s2(sV.in() );
    if (s1 != s2) {
        throw RecycleBroker::NoPermission();
    }

    // Log on
    m_customer_id    = customer_id;
    m_current_status = RecycleBroker::Selling::LOGGED_ON;

    return refV._retn();
}


// create_waste_item() -- Implements IDL operation "RecycleBroker::Selling::create_waste_item".
//
RecycleBroker::WasteItem_ptr
RecycleBroker_Selling_i::create_waste_item(
    const RecycleBroker::WasteItemDetails& initialData,
    RecycleBroker::KeyType_out      wasteitem_id
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NotLoggedOn
))
{
    cout << "RecycleBroker_Selling_i::create_waste_item(): called." << endl;

    if (m_current_status != RecycleBroker::Selling::LOGGED_ON) {
        throw RecycleBroker::NotLoggedOn();
    }

    // Create the waste item
    if (!m_wasteitem_admin_iP) { _init_wasteitem_admin(); }
    RecycleBroker::WasteItem_var refV;
    refV = m_wasteitem_admin_iP->create(initialData, wasteitem_id);
    
    return refV._retn();
}


// log_off() -- Implements IDL operation "RecycleBroker::Selling::log_off".
void
RecycleBroker_Selling_i::log_off() IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NotLoggedOn
))
{
    cout << "RecycleBroker_Selling_i::log_off(): called." << endl;

    if (m_current_status != RecycleBroker::Selling::LOGGED_ON) {
        throw RecycleBroker::NotLoggedOn();
    }
    else {
        m_current_status = RecycleBroker::Selling::LOGGED_OFF;
    }
}


// current_status() -- Accessor for IDL attribute "RecycleBroker::Selling::current_status".
//
RecycleBroker::Selling::Status
RecycleBroker_Selling_i::current_status() IT_THROW_DECL((CORBA::SystemException))
{
    return m_current_status;
}

// Private functions
void
RecycleBroker_Selling_i::_init_customer_admin()
{
    if (!m_customer_admin_iP) {
        m_customer_admin_iP = new RecycleBroker_CustomerAdmin_i(
                                       1,            // hasAdminPrivileges
                                       m_branch_id
                               );
    }
}

void
RecycleBroker_Selling_i::_init_wasteitem_admin()
{
    if (m_current_status != RecycleBroker::Selling::LOGGED_ON) {
        cerr << "error: must be logged on to use WasteItemAdmin" << endl;
        return;
    }
    if (!m_wasteitem_admin_iP) {
        m_wasteitem_admin_iP = new RecycleBroker_WasteItemAdmin_i(
                                       1,            // hasAdminPrivileges
                                       m_branch_id,
                                       m_customer_id
                               );
    }
}

PortableServer::POA_ptr
RecycleBroker_Selling_i::_default_POA()
{
    CORBA::String_var rep_idV = RecycleBroker::_tc_Selling->id();
    return POAMapper::getPOA(rep_idV.in() );
}

