// Class: RecycleBroker_WasteItemAdmin_i
// A POA servant which implements of the RecycleBroker::WasteItemAdmin interface
//

#ifndef RECYCLEBROKER_WASTEITEMADMIN_I_H_
#define RECYCLEBROKER_WASTEITEMADMIN_I_H_

#include "RecycleBrokerS.hh"


class RecycleBroker_WasteItemAdmin_i :
    public virtual PortableServer::RefCountServantBase,
    public virtual POA_RecycleBroker::WasteItemAdmin
{
public:
    RecycleBroker_WasteItemAdmin_i(
        CORBA::Boolean          hasAdminPrivileges,
        RecycleBroker::KeyType  branch_id,
        RecycleBroker::KeyType  customer_id
    );

    RecycleBroker_WasteItemAdmin_i(
        RecycleBroker::KeyType  branch_id
    );

    virtual ~RecycleBroker_WasteItemAdmin_i();

    // _create() -- create a new servant.
    // Hides the difference between direct inheritance and tie servants.
    //    
    static POA_RecycleBroker::WasteItemAdmin*
    _create(
        CORBA::Boolean          hasAdminPrivileges,
        RecycleBroker::KeyType  branch_id,
        RecycleBroker::KeyType  customer_id
    );


    // IDL operations
    //
    virtual RecycleBroker::WasteItem_ptr
    create(
        const RecycleBroker::WasteItemDetails& initialData,
        RecycleBroker::KeyType_out      wasteitem_id
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NoPermission
    ));

    virtual RecycleBroker::WasteItem_ptr
    find(
        RecycleBroker::KeyType          wasteitem_id
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotFound
    ));

    virtual RecycleBroker::WasteItemIdSeq*
    find_by_waste(
        RecycleBroker::WasteType        waste
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotFound
    ));

    virtual RecycleBroker::WasteItemIdSeq*
    find_by_branch(
        RecycleBroker::KeyType          branch_id
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotFound
    ));

    virtual RecycleBroker::WasteItemIdSeq*
    find_all() IT_THROW_DECL((
        CORBA::SystemException
    ));

    virtual RecycleBroker::WasteItemDetailsSeq*
    get_details(
        const RecycleBroker::WasteItemIdSeq& id_seq
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotFound
    ));

    virtual PortableServer::POA_ptr
    _default_POA();

private: 
    // Private members
    CORBA::Boolean               m_hasAdminPrivileges;
    RecycleBroker::KeyType       m_branch_id;
    RecycleBroker::KeyType       m_customer_id;

    // The following are not implemented
    RecycleBroker_WasteItemAdmin_i(const RecycleBroker_WasteItemAdmin_i &);
    RecycleBroker_WasteItemAdmin_i& operator=(const RecycleBroker_WasteItemAdmin_i &);
}; 

#endif
