//----------------------------------------------
// Implementation of 'ToyDB' class
//----------------------------------------------

#include "ToyDB.h"
#include <iostream.h>
#include <fstream>
#include <string>

ToyDB::ToyDB()
  :
    m_wasteitem_key_counter(0),
    m_customer_key_counter(0)
{
    _init_customer_table();
    _init_wasteitem_table();
}

ToyDB::~ToyDB()
{  }


// Return value is 'customer_id'
// A return value of '-1' indicates function failure.
RecycleBroker::KeyType
ToyDB::create_customer_details(
    const RecycleBroker::CustomerDetails& initialData,
    RecycleBroker::KeyType                branch_id
)
{
    cout << "ToyDB::create_customer_details() called:" << endl;

    RecycleBroker::CustomerDetailsFull fullData;
    fullData.public_details = initialData;
    fullData.branch_id      = branch_id;
    fullData.customer_id    = m_customer_key_counter;
    m_customer_table[m_customer_key_counter] = fullData;

    return (RecycleBroker::KeyType) m_customer_key_counter++;
}
    
// Caller is responsible for deleting returned 'dataP' parameter.
CORBA::Boolean
ToyDB::find_customer_details(
    RecycleBroker::KeyType                customer_id,
    RecycleBroker::CustomerDetailsFull*&  dataP
)
{
    cout << "ToyDB::find_customer_details() called:" << endl;

    CustomerTableType::iterator p = m_customer_table.find(customer_id);
    if (p != m_customer_table.end() ) {
        // Customer found.
        dataP = new RecycleBroker::CustomerDetailsFull();
        *dataP = p->second;
        return 1;
    }
    else {
        // Customer not found.
        dataP = 0;
        return 0;
    }
}

RecycleBroker::CustomerIdSeq*
ToyDB::find_customer_details_by_name(
    const char*               name
)
{
    RecycleBroker::CustomerIdSeq* idSeqP
        = new RecycleBroker::CustomerIdSeq(1);

    CORBA::ULong count = 0;
    CustomerTableType::iterator p = m_customer_table.begin();
    while (p != m_customer_table.end() ) {
        if (_findMatchingName(name, (p->second).public_details.name) ) {
            idSeqP->length(count+1);
            (*idSeqP)[count++] = (p->second).customer_id;
        }
        ++p;
    }

    return idSeqP;
}

CORBA::Boolean
ToyDB::update_customer_details(
    RecycleBroker::KeyType                customer_id,
    const RecycleBroker::CustomerDetails& data
)
{
    cout << "ToyDB::update_customer_details() called:" << endl;

    CustomerTableType::iterator p = m_customer_table.find(customer_id);
    if (p != m_customer_table.end() ) {
        // Customer found.
        (p->second).public_details = data;
        return 1;
    }
    else {
        // Customer not found.
        return 0;
    }
}
    
// Return value is 'wasteitem_id'
// A return value of '-1' indicates function failure.
RecycleBroker::KeyType
ToyDB::create_wasteitem_details(
    const RecycleBroker::WasteItemDetails& initialData,
    RecycleBroker::KeyType                 branch_id,
    RecycleBroker::KeyType                 customer_id
)
{
    cout << "ToyDB::create_wasteitem_details() called:" << endl;

    RecycleBroker::WasteItemDetailsFull fullData;
    fullData.public_details = initialData;
    fullData.branch_id      = branch_id;
    fullData.customer_id    = customer_id;
    fullData.wasteitem_id   = m_wasteitem_key_counter;
    m_wasteitem_table[m_wasteitem_key_counter] = fullData;

    return (RecycleBroker::KeyType) m_wasteitem_key_counter++;
}
    
// Caller is responsible for deleting returned 'dataP' parameter.
CORBA::Boolean
ToyDB::find_wasteitem_details(
    RecycleBroker::KeyType                 wasteitem_id,
    RecycleBroker::WasteItemDetailsFull*&  dataP
)
{
    cout << "ToyDB::find_wasteitem_details() called:" << endl;

    WasteItemTableType::iterator p = m_wasteitem_table.find(wasteitem_id);
    if (p != m_wasteitem_table.end() ) {
        // WasteItem found.
        dataP = new RecycleBroker::WasteItemDetailsFull();
        *dataP = p->second;
        return 1;
    }
    else {
        // WasteItem not found.
        dataP = 0;
        return 0;
    }
}

CORBA::Boolean
ToyDB::find_wasteitem_details(
    const RecycleBroker::WasteItemIdSeq&   id_seq,
    RecycleBroker::WasteItemDetailsSeq*&   dataP
)
{
    cout << "ToyDB::find_wasteitem_details() called:" << endl;

    RecycleBroker::WasteItemDetailsSeq_var dataV
        = new RecycleBroker::WasteItemDetailsSeq(id_seq.length());
    dataV->length(0);

    int found_count = 0;
    for (CORBA::ULong u=0; u < id_seq.length(); u++)
    {
        WasteItemTableType::iterator p = m_wasteitem_table.find(id_seq[u]);
        if (p != m_wasteitem_table.end() ) {
            // WasteItem found.
            dataV->length(found_count+1);
            dataV[found_count] = (p->second).public_details;
            found_count++;
        }
        else {
            // WasteItem not found.
            return 0;
        }
    }
    dataP = dataV._retn();
    return 1;
}

CORBA::Boolean
ToyDB::find_wasteitem_by_waste(
    RecycleBroker::WasteType               waste,
    RecycleBroker::WasteItemIdSeq*&        id_seqP
)
{
    id_seqP = new RecycleBroker::WasteItemIdSeq(1);

    CORBA::ULong count = 0;
    WasteItemTableType::iterator p = m_wasteitem_table.begin();
    while (p != m_wasteitem_table.end() ) {
        if ( waste == (p->second).public_details.waste ) {
            id_seqP->length(count+1);
            (*id_seqP)[count++] = (p->second).wasteitem_id;
        }
        ++p;
    }

    return (id_seqP->length() > 0);
}

CORBA::Boolean
ToyDB::find_wasteitem_by_branch(
    RecycleBroker::KeyType                 branch_id,
    RecycleBroker::WasteItemIdSeq*&        id_seqP
)
{
    id_seqP = new RecycleBroker::WasteItemIdSeq(1);

    CORBA::ULong count = 0;
    WasteItemTableType::iterator p = m_wasteitem_table.begin();
    while (p != m_wasteitem_table.end() ) {
        if ( branch_id == (p->second).branch_id ) {
            id_seqP->length(count+1);
            (*id_seqP)[count++] = (p->second).wasteitem_id;
        }
        ++p;
    }

    return (id_seqP->length() > 0);
}

CORBA::Boolean
ToyDB::find_all_wasteitem(
    RecycleBroker::WasteItemIdSeq*&        id_seqP
)
{
    cout << "ToyDB::find_all_wasteitem() called:" << endl;

    id_seqP = new RecycleBroker::WasteItemIdSeq(10);

    CORBA::ULong count = 0;
    WasteItemTableType::iterator p = m_wasteitem_table.begin();
    while (p != m_wasteitem_table.end() ) {
        id_seqP->length(count+1);
        (*id_seqP)[count] = p->first;
        ++count;
        ++p;
    }

    return count > 0;
}

CORBA::Boolean
ToyDB::update_wasteitem_details(
    RecycleBroker::KeyType                 wasteitem_id,
    const RecycleBroker::WasteItemDetails& data
)
{
    cout << "ToyDB::update_wasteitem_details() called:" << endl;

    WasteItemTableType::iterator p = m_wasteitem_table.find(wasteitem_id);
    if (p != m_wasteitem_table.end() ) {
        // WasteItem found.
        (p->second).public_details = data;
        return 1;
    }
    else {
        // WasteItem not found.
        return 0;
    }
}

// Private functions
CORBA::Boolean
ToyDB::_findMatchingName(const char* search_str, const char* str)
{
    std::string s(str);
    return (s.find(search_str) != std::string::npos);
}
    
void
ToyDB::_init_customer_table()
{
    RecycleBroker::CustomerDetailsFull tmp;

    std::ifstream dbs("customer.db");
    if (!dbs) {
        cerr << "error: cannot open 'customer.db' file." << endl;
    }

    char buf[1024];
    int  key_val;
    dbs >> m_customer_key_counter;
    dbs.ignore(1024, '\n');
    while (!dbs.eof() ) {
        if ( !dbs.getline(buf, 1024, ':') )       { break; }
        tmp.public_details.name  = CORBA::string_dup(buf);

        if ( !dbs.getline(buf, 1024, ':') )       { break; }
        tmp.public_details.address  = CORBA::string_dup(buf);

        if ( !dbs.getline(buf, 1024, ':') )       { break; }
        tmp.public_details.email_address  = CORBA::string_dup(buf);

        if ( !dbs.getline(buf, 1024, ':') )       { break; }
        tmp.public_details.password  = CORBA::string_dup(buf);

        if ( !(dbs >> key_val) )              { break; }
        tmp.branch_id = (RecycleBroker::KeyType) key_val;
        if ( !dbs.ignore(1024, ':') )               { break; }

        if ( !(dbs >> key_val) )              { break; }
        tmp.customer_id = (RecycleBroker::KeyType) key_val;
        dbs.ignore(1024, '\n');

        m_customer_table[tmp.customer_id] = tmp;
    }
}

void
ToyDB::_init_wasteitem_table()
{
    RecycleBroker::WasteItemDetailsFull tmp;

    std::ifstream dbs("wasteitem.db");
    if (!dbs) {
        cerr << "error: cannot open 'wasteitem.db' file." << endl;
    }

    int  int_value;
    dbs >> m_wasteitem_key_counter;
    dbs.ignore(1024, '\n');
    while (!dbs.eof() ) {
        if ( !(dbs >> int_value) )              { break; }
        tmp.public_details.waste = (RecycleBroker::WasteType) int_value;
        if ( !dbs.ignore(1024, ':') )               { break; }

        if ( !(dbs >> tmp.public_details.quantity) )  { break; }
        if ( !dbs.ignore(1024, ':') )                 { break; }

        if ( !(dbs >> tmp.public_details.price_per_kilo) )  { break; }
        if ( !dbs.ignore(1024, ':') )               { break; }

        if ( !(dbs >> int_value) )              { break; }
        tmp.branch_id = (RecycleBroker::KeyType) int_value;
        if ( !dbs.ignore(1024, ':') )               { break; }

        if ( !(dbs >> int_value) )              { break; }
        tmp.customer_id = (RecycleBroker::KeyType) int_value;
        if ( !dbs.ignore(1024, ':') )               { break; }

        if ( !(dbs >> int_value) )              { break; }
        tmp.wasteitem_id = (RecycleBroker::KeyType) int_value;
        dbs.ignore(1024, '\n');

        m_wasteitem_table[tmp.wasteitem_id] = tmp;
    }
}

