#ifndef _TOYDB_H_
#define _TOYDB_H_

#include "DBBase.h"

//---------------------
// An implementation of the 'DBBase' abstract class
//---------------------

// Use the Standard Template Library (STL)
#include <map>

class ToyDB : public DBBase {

public:
    // Return value is 'customer_id'
    // A return value of '-1' indicates function failure.
    virtual RecycleBroker::KeyType
    create_customer_details(
        const RecycleBroker::CustomerDetails& initialData,
	RecycleBroker::KeyType                branch_id
    );
    
    // Caller is responsible for deleting returned 'dataP' parameter.
    virtual CORBA::Boolean
    find_customer_details(
	RecycleBroker::KeyType                customer_id,
        RecycleBroker::CustomerDetailsFull*&  dataP
    );

    virtual RecycleBroker::CustomerIdSeq*
    find_customer_details_by_name(
	const char*               name
    );

    virtual CORBA::Boolean
    update_customer_details(
	RecycleBroker::KeyType                customer_id,
        const RecycleBroker::CustomerDetails& data
    );
    
    // Return value is 'wasteitem_id'
    // A return value of '-1' indicates function failure.
    virtual RecycleBroker::KeyType
    create_wasteitem_details(
        const RecycleBroker::WasteItemDetails& initialData,
	RecycleBroker::KeyType                 branch_id,
	RecycleBroker::KeyType                 customer_id
    );
    
    // Caller is responsible for deleting returned 'dataP' parameter.
    virtual CORBA::Boolean
    find_wasteitem_details(
	RecycleBroker::KeyType                 wasteitem_id,
        RecycleBroker::WasteItemDetailsFull*&  dataP
    );

    virtual CORBA::Boolean
    find_wasteitem_details(
        const RecycleBroker::WasteItemIdSeq&   id_seq,
        RecycleBroker::WasteItemDetailsSeq*&   dataP
    );

    virtual CORBA::Boolean
    find_wasteitem_by_waste(
        RecycleBroker::WasteType               waste,
        RecycleBroker::WasteItemIdSeq*&        id_seqP
    );

    virtual CORBA::Boolean
    find_wasteitem_by_branch(
        RecycleBroker::KeyType                 branch_id,
        RecycleBroker::WasteItemIdSeq*&        id_seqP
    );

    virtual CORBA::Boolean
    find_all_wasteitem(
        RecycleBroker::WasteItemIdSeq*&        id_seqP
    );

    virtual CORBA::Boolean
    update_wasteitem_details(
	RecycleBroker::KeyType                 wasteitem_id,
        const RecycleBroker::WasteItemDetails& data
    );
    
    // Constructor
    ToyDB();

    // Destructor
    virtual ~ToyDB();

private:
    typedef std::map<RecycleBroker::KeyType,
                     RecycleBroker::WasteItemDetailsFull>
            WasteItemTableType;

    typedef std::map<RecycleBroker::KeyType, RecycleBroker::CustomerDetailsFull>
            CustomerTableType;

    // Private Member Variables
    int                           m_wasteitem_key_counter;
    int                           m_customer_key_counter;
    WasteItemTableType            m_wasteitem_table;
    CustomerTableType             m_customer_table;

    // Private functions
    CORBA::Boolean _findMatchingName(const char* search_str, const char* str);
    void _init_customer_table();
    void _init_wasteitem_table();
    
    // Not defined in this class
    ToyDB(const ToyDB &);
    ToyDB & operator=(const ToyDB &);
};

#endif
