#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string>
#include "RecycleBroker.hh"


// global_orb -- make ORB global so all code can find it.
CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// read_reference() -- read an object reference from file.
static 
CORBA::Object_ptr
read_reference(
    const char*      file
)
{
    cout << "Reading stringified object reference from " << file << endl;
    ifstream ifs(file);
    CORBA::String_var str;
    ifs >> str;
    if (!ifs) {
        cerr << "Error reading object reference from " << file << endl;
        return CORBA::Object::_nil();
    }
    return global_orb->string_to_object(str);
}

static void
print_wastetype_menu()
{
    cout << "\t0\tBROWN_GLASS" << endl;
    cout << "\t1\tGREEN_GLASS" << endl;
    cout << "\t2\tCLEAR_GLASS" << endl;
    cout << "\t3\tSCRAP_STEEL" << endl;
    cout << "\t4\tALUMINIUM_CANS" << endl;
    cout << "\t5\tPLASTIC_BOTTLES" << endl;
    cout << "\t6\tWASTE_PAPER" << endl;
}

static void
print_wasteitem_details(
    const RecycleBroker::WasteItemDetails& details,
    const char* indent
)
{
    cout << indent << "waste:\t\t";
    switch (details.waste) {
        case RecycleBroker::BROWN_GLASS:
            cout << "BROWN_GLASS" << endl;
            break;
        case RecycleBroker::GREEN_GLASS:
            cout << "GREEN_GLASS" << endl;
            break;
        case RecycleBroker::CLEAR_GLASS:
            cout << "CLEAR_GLASS" << endl;
            break;
        case RecycleBroker::SCRAP_STEEL:
            cout << "SCRAP_STEEL" << endl;
            break;
        case RecycleBroker::ALUMINIUM_CANS:
            cout << "ALUMINIUM_CANS" << endl;
            break;
        case RecycleBroker::PLASTIC_BOTTLES:
            cout << "PLASTIC_BOTTLES" << endl;
            break;
        case RecycleBroker::WASTE_PAPER:
            cout << "WASTE_PAPER" << endl;
            break;
        default:
            cout << "<unknown>" << endl;
            break;
    }
    cout << indent << "quantity:\t" << details.quantity << endl;
    cout << indent << "price_per_kilo:\t" << details.price_per_kilo << endl;
}


int
main(int argc, char **argv)
{
    int exit_status = 0;        
    try
    {
        CORBA::Object_var tmp_ref;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

        // Get reference to 'RecycleBroker::BranchOffice'.
        tmp_ref = read_reference("RecycleBroker_BranchOffice.ref");
        RecycleBroker::BranchOffice_var branchOfficeV =
            RecycleBroker::BranchOffice::_narrow(tmp_ref);
        if (CORBA::is_nil(branchOfficeV.in() )) 
        {
            cerr << "Could not narrow reference to interface "
                 << "RecycleBroker::BranchOffice" << endl;
            exit(1);
        }

        // Get 'Browsing' and 'WasteItemAdmin' interfaces.
        RecycleBroker::Browsing_var browsingV
            = branchOfficeV->get_browsing();
        RecycleBroker::WasteItemAdmin_var wasteitem_adminV
            = browsingV->get_waste_item_admin();

        cout << "List of all available waste items:" << endl;
        RecycleBroker::WasteItemIdSeq_var id_seqV
            = wasteitem_adminV->find_all();
        RecycleBroker::WasteItemDetailsSeq_var detailsSeqV
            = wasteitem_adminV->get_details(id_seqV.in() );

        for (CORBA::ULong j=0; j < id_seqV->length(); j++)
        {
            cout << "wasteitem_id = " << id_seqV[j] << endl;
            print_wasteitem_details(detailsSeqV[j], "\t");
        }
    }
    catch (RecycleBroker::NoPermission& )
    {
        cerr << "error: Logon failed." << endl;
    }
    catch (RecycleBroker::NotFound& )
    {
        cerr << "error: Could not find this item." << endl;
    }
    catch (RecycleBroker::InsufficientQuantity& )
    {
        cerr << "error: Insufficient quantity of this item available." << endl;
    }
    catch (CORBA::Exception &ex) 
    {
        cerr << "error: Unexpected CORBA exception: " << ex << endl;
        exit_status = 1;
    }

    // Ensure that the ORB is properly shutdown and cleaned up.
    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return exit_status;
}


