#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string>
#include "RecycleBroker.hh"


// global_orb -- make ORB global so all code can find it.
CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// read_reference() -- read an object reference from file.
static 
CORBA::Object_ptr
read_reference(
    const char*      file
)
{
    cout << "Reading stringified object reference from " << file << endl;
    ifstream ifs(file);
    CORBA::String_var str;
    ifs >> str;
    if (!ifs) {
        cerr << "Error reading object reference from " << file << endl;
        return CORBA::Object::_nil();
    }
    return global_orb->string_to_object(str);
}

static void
print_wastetype_menu()
{
    cout << "\t0\tBROWN_GLASS" << endl;
    cout << "\t1\tGREEN_GLASS" << endl;
    cout << "\t2\tCLEAR_GLASS" << endl;
    cout << "\t3\tSCRAP_STEEL" << endl;
    cout << "\t4\tALUMINIUM_CANS" << endl;
    cout << "\t5\tPLASTIC_BOTTLES" << endl;
    cout << "\t6\tWASTE_PAPER" << endl;
}

static void
print_wasteitem_details(
    const RecycleBroker::WasteItemDetails& details,
    const char* indent
)
{
    cout << indent << "waste:\t\t";
    switch (details.waste) {
        case RecycleBroker::BROWN_GLASS:
            cout << "BROWN_GLASS" << endl;
            break;
        case RecycleBroker::GREEN_GLASS:
            cout << "GREEN_GLASS" << endl;
            break;
        case RecycleBroker::CLEAR_GLASS:
            cout << "CLEAR_GLASS" << endl;
            break;
        case RecycleBroker::SCRAP_STEEL:
            cout << "SCRAP_STEEL" << endl;
            break;
        case RecycleBroker::ALUMINIUM_CANS:
            cout << "ALUMINIUM_CANS" << endl;
            break;
        case RecycleBroker::PLASTIC_BOTTLES:
            cout << "PLASTIC_BOTTLES" << endl;
            break;
        case RecycleBroker::WASTE_PAPER:
            cout << "WASTE_PAPER" << endl;
            break;
        default:
            cout << "<unknown>" << endl;
            break;
    }

    cout << indent << "quantity:\t" << details.quantity << endl;

    cout << indent << "price_per_kilo:\t" << details.price_per_kilo << endl;
}


int
main(int argc, char **argv)
{
    int exit_status = 0;        
    try
    {
        CORBA::Object_var tmp_ref;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

        // Get reference to 'RecycleBroker::BranchOffice'.
        tmp_ref = read_reference("RecycleBroker_BranchOffice.ref");
        RecycleBroker::BranchOffice_var branchOfficeV =
            RecycleBroker::BranchOffice::_narrow(tmp_ref);
        if (CORBA::is_nil(branchOfficeV.in() )) 
        {
            cerr << "Could not narrow reference to interface "
                 << "RecycleBroker::BranchOffice" << endl;
            exit(1);
        }

        // Use 'RecycleBroker::Buying' interface.
        tmp_ref = branchOfficeV->get_buying();
        RecycleBroker::Buying_var buyingV =
            RecycleBroker::Buying::_narrow(tmp_ref);
        if (CORBA::is_nil(buyingV.in() )) 
        {
            cerr << "Could not narrow reference to interface "
                 << "RecycleBroker::Buying" << endl;
            exit(1);
        }

        // 1. Log on
        cout << "Log on" << endl;
        char s[256];
        cout << "\tname:\t";
        cin.getline(s, 256);
        CORBA::String_var logon_nameV = CORBA::string_dup(s);
        cout << "\tpasswd:\t";
        cin.getline(s, 256);
        CORBA::String_var logon_passwdV = CORBA::string_dup(s);
        
        buyingV->log_on(logon_nameV.in(), logon_passwdV.in() );
        

        // 2. Select waste item to purchase.
        cout << endl << "Buy a waste item" << endl << endl;
        RecycleBroker::KeyType wasteitem_id;
        cout << "enter wasteitem_id:\t";
        cin >> wasteitem_id;
        CORBA::Long  quantity;
        cout << "enter quantity (kilos):\t";
        cin >> quantity;

        // 3. Show waste item details.
        RecycleBroker::Browsing_var browsingV;
        browsingV = branchOfficeV->get_browsing();
        RecycleBroker::WasteItemAdmin_var wasteitem_adminV;
        wasteitem_adminV = browsingV->get_waste_item_admin();
        RecycleBroker::WasteItem_var wasteitemV;
        wasteitemV = wasteitem_adminV->find(wasteitem_id);

        cout << endl << "Before purchase" << endl;
        RecycleBroker::WasteItemDetails_var detailsV
            = wasteitemV->get_details();
        print_wasteitem_details(detailsV.in(), "\t");


        // 4. Buy waste item.
        buyingV->buy_item(wasteitem_id, quantity);
        cout << endl
             << "Bought " << quantity
             << " kilos of waste item with wasteitem_id = "
             << wasteitem_id << endl;

        // 5. Show waste item details again.
        cout << endl << "After purchase" << endl;
        detailsV = wasteitemV->get_details();
        print_wasteitem_details(detailsV.in(), "\t");

        // 6. Log off
        buyingV->log_off();
        cout << endl << "Logged off" << endl;
    }
    catch (RecycleBroker::NoPermission& )
    {
        cerr << "error: Logon failed." << endl;
    }
    catch (RecycleBroker::NotFound& )
    {
        cerr << "error: Could not find this item." << endl;
    }
    catch (RecycleBroker::InsufficientQuantity& )
    {
        cerr << "error: Insufficient quantity of this item available." << endl;
    }
    catch (CORBA::Exception &ex) 
    {
        cerr << "error: Unexpected CORBA exception: " << ex << endl;
        exit_status = 1;
    }

    // Ensure that the ORB is properly shutdown and cleaned up.
    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return exit_status;
}


