#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string>
#include "RecycleBroker.hh"


// global_orb -- make ORB global so all code can find it.
CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// read_reference() -- read an object reference from file.
// Reads a stringified object reference from file and returns a real
// object reference.
// Caller must release the returned reference or assign it to a _var. 
// On error returns a nil object reference.
//
static 
CORBA::Object_ptr
read_reference(
    const char*      file
)
{
    cout << "Reading stringified object reference from " << file << endl;
    ifstream ifs(file);
    CORBA::String_var str;
    ifs >> str;
    if (!ifs) {
        cerr << "Error reading object reference from " << file << endl;
        return CORBA::Object::_nil();
    }
    return global_orb->string_to_object(str);
}


// main() -- the main client program.
//
int
main(int argc, char **argv)
{
    int exit_status = 0;        
    try
    {
        CORBA::Object_var tmp_ref;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

        // Get reference to 'RecycleBroker::BranchOffice'.
        tmp_ref = read_reference("RecycleBroker_BranchOffice.ref");
        RecycleBroker::BranchOffice_var branchOfficeV =
            RecycleBroker::BranchOffice::_narrow(tmp_ref);
        if (CORBA::is_nil(branchOfficeV.in() )) 
        {
            cerr << "Could not narrow reference to interface "
                 << "RecycleBroker::BranchOffice" << endl;
        }

        // Exercise 'RecycleBroker::CustomerAdmin' interface.
        tmp_ref = branchOfficeV->get_customer_admin();
        RecycleBroker::CustomerAdmin_var customer_adminV =
            RecycleBroker::CustomerAdmin::_narrow(tmp_ref);
        if (CORBA::is_nil(customer_adminV.in() )) 
        {
            cerr << "Could not narrow reference to interface "
                 << "RecycleBroker::CustomerAdmin" << endl;
            exit(1);
        }
        cout << "Creating new customer ..." << endl;
        RecycleBroker::CustomerDetails data;
        RecycleBroker::KeyType         customer_id;
        char s[256];
        cout << "enter name:\t\t";
        cin.getline(s, 256);
        data.name = CORBA::string_dup(s);
        cout << "enter address:\t\t";
        cin.getline(s, 256);
        data.address = CORBA::string_dup(s);
        cout << "enter email address:\t";
        cin.getline(s, 256);
        data.email_address = CORBA::string_dup(s);
        cout << "enter password:\t\t";
        cin.getline(s, 256);
        data.password = CORBA::string_dup(s);
        RecycleBroker::Customer_var customerV;
        customerV = customer_adminV->create(data, customer_id);
        cout << endl
             << "Created customer with customer_id = "
             << customer_id << endl;
        try {
            cout << endl << "Checking details:" << endl;
            RecycleBroker::CustomerDetails_var detailsV;
            detailsV = customerV->get_details();
            cout << "\tname:\t\t" << detailsV->name << endl;
            cout << "\taddress:\t" << detailsV->address << endl;
            cout << "\temail_address:\t" << detailsV->email_address << endl;
            cout << "\tpassword:\t" << detailsV->password << endl;
        }
        catch (RecycleBroker::NotFound& ) {
            cerr << "error: customer not found." << endl;
        }
    }
    catch(CORBA::Exception &ex) 
    {
        cerr << "Unexpected CORBA exception: " << ex << endl;
        exit_status = 1;
    }

    // Ensure that the ORB is properly shutdown and cleaned up.
    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return exit_status;
}


