
#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <it_ts/termination_handler.h>
#include <omg/PortableServer.hh>
#include "POAUtility.h"
#include "RecycleBroker_Customer_i.h"
#include "RecycleBroker_WasteItem_i.h"
#include "RecycleBroker_CustomerAdmin_i.h"
#include "RecycleBroker_WasteItemAdmin_i.h"
#include "RecycleBroker_OfficeAdmin_i.h"
#include "RecycleBroker_Browsing_i.h"
#include "RecycleBroker_Selling_i.h"
#include "RecycleBroker_Buying_i.h"
#include "RecycleBroker_Office_i.h"
#include "RecycleBroker_HeadOffice_i.h"
#include "RecycleBroker_BranchOffice_i.h"
#include "POAMapper.h"
#include "ToyDB.h"

#include "NameUtil.h"


// global_orb -- make ORB global so all code can find it.
CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();

// Declare 'global_db'
DBBase* global_db = 0;

// sig_handler() -- handle fatal signals/events.
// On receipt of a signal, we shut down the ORB. This causes 
// ORB::run() to return, and allows the server to shut down
// gracefully.
//
static
void 
sig_handler(
    long sig_type
)
{
    if (!CORBA::is_nil(global_orb))
    {
        global_orb->shutdown(0);
    }
    else 
    {
        exit(0);
    }
}


// write_reference() -- export object reference to file.
void
write_reference(
    CORBA::Object_ptr               ref,
    const char*                     objref_file
)
{
    CORBA::String_var stringified_ref = global_orb->object_to_string(ref);
    cout << "Writing stringified object reference to " << objref_file << endl;
    
    ofstream os(objref_file);
    os << stringified_ref;
    if (!os.good())
    {
        cerr << "Failed to write to " << objref_file << endl;
    }
}


int
main(
    int argc,
    char **argv
)
{
    int exit_status = 0;             // Return code from main.

    // Initialize the database instance
    global_db = new ToyDB();

    // Variables to hold servants.
    //
    // PortableServer::Servant the_RecycleBroker_HeadOffice = 0;
    PortableServer::Servant the_RecycleBroker_BranchOffice = 0;

    try
    {
        CORBA::Object_var tmp_ref;  // For temporary object references.

        // Initialise the ORB and Root POA.
        //
        cout << "Initializing the ORB" << endl;
        global_orb = CORBA::ORB_init(argc, argv);
        IT_TerminationHandler::set_signal_handler(sig_handler);
        tmp_ref = global_orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var root_poa =
            PortableServer::POA::_narrow(tmp_ref);
        assert(!CORBA::is_nil(root_poa));
        PortableServer::POAManager_var root_poa_manager
            = root_poa->the_POAManager();
        assert(!CORBA::is_nil(root_poa_manager));

        // Create a session POA and an entity POA
        //
        PortableServer::POA_var session_poaV =
            POAUtility::create_basic_POA(
                root_poa,               // Parent POA
                root_poa_manager,       // POA Manager
                "session_poa",          // POA name
                0,                      // isMultThreaded
                0                       // isPersistent=0 => session object
            );

        PortableServer::POA_var entity_poaV =
            POAUtility::create_basic_POA(
                root_poa,               // Parent POA
                root_poa_manager,       // POA Manager
                "entity_poa",          // POA name
                0,                      // isMultThreaded
                1                       // isPersistent=1 => entity object
            );

        // Initialize 'POAMapper' hash table.
        CORBA::String_var rep_idV = RecycleBroker::_tc_Office->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_BranchOffice->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_HeadOffice->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_Browsing->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_Selling->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_Buying->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_CustomerAdmin->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_WasteItemAdmin->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_OfficeAdmin->id();
        POAMapper::addPOAEntry(rep_idV.in(), session_poaV);

        rep_idV = RecycleBroker::_tc_Customer->id();
        POAMapper::addPOAEntry(rep_idV.in(), entity_poaV);

        rep_idV = RecycleBroker::_tc_WasteItem->id();
        POAMapper::addPOAEntry(rep_idV.in(), entity_poaV);

        // Create servants and export object references.
        PortableServer::ObjectId_var oid;
        PortableServer::POA_var poa_for_BranchOfficeV;

        // Create a servant for interface RecycleBroker::BranchOffice.
        the_RecycleBroker_BranchOffice =
            RecycleBroker_BranchOffice_i::_create(
                "3 CORBA Lane, New York",  // Office address
                12345                      // Branch id
            );
        poa_for_BranchOfficeV = the_RecycleBroker_BranchOffice->_default_POA();
        oid = poa_for_BranchOfficeV->activate_object(
                  the_RecycleBroker_BranchOffice
              );
        tmp_ref = poa_for_BranchOfficeV->id_to_reference(oid);
        write_reference(tmp_ref,"RecycleBroker_BranchOffice.ref");

        // Export the BranchOffice object reference to the Naming Service
        CORBA::Object_var objV;
	CosNaming::Name_var nameV;
        CosNaming::NamingContextExt_var rootContextExtV;

        try
        {
            objV = global_orb->resolve_initial_references("NameService");
            rootContextExtV = CosNaming::NamingContextExt::_narrow(objV);
        }
        catch (CORBA::SystemException &sysEx) {
            cerr << sysEx;
            return 1;
        } 

	if (CORBA::is_nil(rootContextExtV )) {
	     cerr << "error: rootContextExt is nil" << endl;
	     return 1;
	}

	nameV = rootContextExtV->to_name(
	          "Pure/RecycleBroker/BranchOffice"
		);

        NameUtil::bindObjectPath(rootContextExtV, nameV, tmp_ref);

        // Activate the POA Manager and let the ORB process requests.
        //
        root_poa_manager->activate();
        cout << "Waiting for requests..." << endl;
        global_orb->run();
    }
    catch (CORBA::Exception& e)
    {
        cout << "Unexpected CORBA exception: " << e << endl;
        exit_status = 1;
    }
    // Delete the servants.
    //
    // the_RecycleBroker_HeadOffice->_remove_ref();
    if (the_RecycleBroker_BranchOffice) 
        the_RecycleBroker_BranchOffice->_remove_ref();

    // Ensure that the ORB is properly shutdown and cleaned up.
    //
    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return exit_status;
}
