
#include <stdlib.h>
#include <iostream.h>
#include "BookRepository_BorrowableCollectionImpl.h"

#include <time.h>

// _create() -- create a new servant.
POA_BookRepository::BorrowableCollection*
BookRepository_BorrowableCollectionImpl::_create(
    PortableServer::POA_ptr the_poa
)
{
    return new BookRepository_BorrowableCollectionImpl(the_poa);
}


// Constructor
BookRepository_BorrowableCollectionImpl::BookRepository_BorrowableCollectionImpl(
    PortableServer::POA_ptr the_poa
) :
    BookRepository_CollectionImpl(the_poa)
{
}


// Destructor.
BookRepository_BorrowableCollectionImpl::~BookRepository_BorrowableCollectionImpl()
{
}

BookRepository::Date&
BookRepository_BorrowableCollectionImpl::current_date()
{
    time_t utc_time = time(0);
    tm *   time_details = gmtime(&utc_time);

    BookRepository::Date* todayP = new BookRepository::Date();
    todayP->day   = (CORBA::Short) time_details->tm_mday;
    todayP->month = (CORBA::Short) time_details->tm_mon + 1;
    todayP->year  = (CORBA::Short) time_details->tm_year + 1900;
    return *todayP;
}

void
BookRepository_BorrowableCollectionImpl::increment_date(
    BookRepository::Date& given_date,
    int days
)
{
    static const int seconds_per_day = 24*60*60;

    tm time_details;
    time_details.tm_sec   = 0;
    time_details.tm_min   = 0;
    time_details.tm_hour  = 2;
    time_details.tm_mday  = given_date.day;
    time_details.tm_mon   = given_date.month - 1;
    time_details.tm_year  = given_date.year - 1900;
    time_details.tm_isdst = -1;
    time_t utc_time = mktime(&time_details);

    utc_time += days*seconds_per_day;

    tm *   new_details = gmtime(&utc_time);
    given_date.day   = (CORBA::Short) new_details->tm_mday;
    given_date.month = (CORBA::Short) new_details->tm_mon + 1;
    given_date.year  = (CORBA::Short) new_details->tm_year + 1900;
}

// borrow_book() 
void
BookRepository_BorrowableCollectionImpl::borrow_book(
    BookRepository::ISBN            book_id,
    const BookRepository::PersonName& borrower,
    BookRepository::Date_out        return_date
) throw (
    CORBA::SystemException,
    BookRepository::BorrowableCollection::Unavailable
)
{
    // Check availability of book:
    Borrower_DB::LoanRecord loan;

    if ( m_borrower_db.get(book_id, loan) ) {
        throw BookRepository::BorrowableCollection::Unavailable(
                  loan.return_date
              );
    }

    return_date = current_date();
    increment_date(return_date, 14);

    loan.borrower = borrower;
    loan.return_date = return_date;
    m_borrower_db.put(book_id, loan);
}

