
#include "Collection_DB.h"
#include <fstream>
#include <string>


Collection_DB::Collection_DB()
{
    BookRepository::BookDetails tmp;

    std::ifstream dbs("collection.db");
    if (!dbs) {
        cerr << "error: cannot open 'collection.db' file." << endl;
    }

    char buf[1024];
    while (!dbs.eof() ) {
        if ( !dbs.ignore(1024, ':') )             { break; }

        if ( !dbs.getline(buf, 1024, ':') )       { break; }
        tmp.author.first_name  = CORBA::string_dup(buf);

        if ( !dbs.getline(buf, 1024, ':') )       { break; }
        tmp.author.second_name = CORBA::string_dup(buf);

        if ( !dbs.getline(buf, 1024, ':') )       { break; }
        tmp.title              = CORBA::string_dup(buf);

        if ( !(dbs >> tmp.book_id) )              { break; }
        if ( !dbs.ignore(1024, ':') )             { break; }

        if ( !(dbs >> tmp.publication_date.day) ) { break; }
        if ( !dbs.ignore(1024, ':') )             { break; }

        if ( !(dbs >> tmp.publication_date.month) ) { break; }
        if ( !dbs.ignore(1024, ':') )             { break; }

        if ( !(dbs >> tmp.publication_date.year) )  { break; }

        m_book_table[tmp.book_id] = tmp;
    }
}

CORBA::Boolean
Collection_DB::foundMatchingTitle(const char* search_str, const char* title)
{
    std::string s(title);
    return (s.find(search_str) != std::string::npos);
}

CORBA::Boolean
Collection_DB::is_in_collection(CORBA::Long book_id)
{
    return (m_book_table.count(book_id) > 0);
}

BookRepository::BookDetails*
Collection_DB::get(CORBA::Long book_id)
{
    Collection_DB::TableType::iterator m = m_book_table.find(book_id);
    BookRepository::BookDetails* detailsP = new BookRepository::BookDetails();

    if ( m != m_book_table.end() ) {
        // Make a deep copy of 'm_book_table[book_id]'
        *detailsP = m->second;
    }

    return detailsP;
}
    
BookRepository::BookDetailsSeq*
Collection_DB::getByTitle(const char * title)
{
    BookRepository::BookDetailsSeq* detailsSeqP
        = new BookRepository::BookDetailsSeq(1);

    CORBA::ULong count = 0;
    Collection_DB::TableType::iterator m = m_book_table.begin();
    while (m != m_book_table.end() ) {
        if (foundMatchingTitle(title, m->second.title) ) {
            detailsSeqP->length(count+1);
            (*detailsSeqP)[count++] = m->second;
        }
        ++m;
    }

    return detailsSeqP;
}



