#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <it_ts/termination_handler.h>
#include <omg/PortableServer.hh>
#include <omg/CosNaming.hh>
#include "BookRepository_CollectionImpl.h"
#include "BookRepository_SearchableCollectionImpl.h"
#include "BookRepository_BorrowableCollectionImpl.h"
#include "BookRepository_FlexibleCollectionImpl.h"


CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// sig_handler() -- handle fatal signals/events.
static
void 
sig_handler(
    long sig_type
)
{
    if (!CORBA::is_nil(global_orb))
    {
        global_orb->shutdown(0);
    }
    else 
    {
        exit(0);
    }
}


// write_reference() -- export object reference to file.
void
write_reference(
    CORBA::Object_ptr               refP,
    const char*                     objref_file
)
{
    CORBA::String_var stringified_ref = global_orb->object_to_string(refP);
    cout << "Writing stringified object reference to " << objref_file << endl;
    
    ofstream os(objref_file);
    os << stringified_ref;
    if (!os.good())
    {
        cerr << "Failed to write to " << objref_file << endl;
    }
}


// create_simple_poa() -- Create a POA for simple servant management.
PortableServer::POA_ptr
create_simple_poa(
    const char*                    poa_name,
    PortableServer::POA_ptr        parent_poa,
    PortableServer::POAManager_ptr poa_manager
)
{
    // Create a policy list. Policies not set in the list get default values.
    CORBA::PolicyList policies;
    policies.length(1);
    int i = 0;

    // Make the POA single threaded.
    policies[i++] = parent_poa->create_thread_policy(
        PortableServer::SINGLE_THREAD_MODEL
    );
    assert(i==1);

    return parent_poa->create_POA(poa_name, poa_manager, policies);
}


int
main(
    int argc,
    char **argv
)
{
    // Variables to hold servants.
    PortableServer::ServantBase_var the_BookRepository_Collection = 0;
    PortableServer::ServantBase_var the_BookRepository_SearchableCollection = 0;
    PortableServer::ServantBase_var the_BookRepository_BorrowableCollection = 0;
    PortableServer::ServantBase_var the_BookRepository_FlexibleCollection = 0;

    try
    {
        CORBA::Object_var objV;  // For temporary object references.

        // Initialise the ORB and Root POA.
        cout << "Initializing the ORB" << endl;
        global_orb = CORBA::ORB_init(argc, argv);
        IT_TerminationHandler::set_signal_handler(sig_handler);
        objV = global_orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var root_poaV
            = PortableServer::POA::_narrow(objV);
        if (CORBA::is_nil(root_poaV.in()) ) {
            cerr << "error: failed to narrow root POA." << endl;
            exit(1);
        }
        PortableServer::POAManager_var root_poa_managerV
            = root_poaV->the_POAManager();
        if (CORBA::is_nil(root_poa_managerV.in()) ) {
            cerr << "error: failed to narrow root POA manager." << endl;
            exit(1);
        }

        // Obtain Naming Service reference.
        CosNaming::NamingContext_var rootContextV;
        objV = global_orb->resolve_initial_references("NameService");
        rootContextV = CosNaming::NamingContext::_narrow(objV);
        if (CORBA::is_nil(rootContextV.in()) ) {
            cerr << "error: failed to narrow root naming context." << endl;
            exit(1);
        }

        // Make sure that a 'BookRepository' naming context exists.
        CosNaming::Name objectName(2);
        objectName.length(1);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        try {
            rootContextV->bind_new_context(objectName);
        }
        catch (CORBA::Exception& ) {  }

        // Create a POA.
        PortableServer::POA_var poaV =
            create_simple_poa("simple_poa", root_poaV, root_poa_managerV);

        // Create servants and export object references.
        PortableServer::ObjectId_var oid;

        // Create a servant for interface BookRepository::Collection.
        the_BookRepository_Collection =
            BookRepository_CollectionImpl::_create(poaV);
        oid = poaV->activate_object(the_BookRepository_Collection.in() );
        objV = poaV->id_to_reference(oid);
        write_reference(objV, "BookRepository_Collection.ref");
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("PlainCollection");
        objectName[1].kind = CORBA::string_dup("");
        rootContextV->rebind(objectName, objV);

        // Create a servant for interface BookRepository::SearchableCollection.
        the_BookRepository_SearchableCollection =
            BookRepository_SearchableCollectionImpl::_create(poaV);
        oid = poaV->activate_object(
                        the_BookRepository_SearchableCollection.in()
                    );
        objV = poaV->id_to_reference(oid);
        write_reference(objV,"BookRepository_SearchableCollection.ref");
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("Searchable");
        objectName[1].kind = CORBA::string_dup("");
        rootContextV->rebind(objectName, objV);

        // Create a servant for interface BookRepository::BorrowableCollection.
        the_BookRepository_BorrowableCollection =
            BookRepository_BorrowableCollectionImpl::_create(poaV);
        oid = poaV->activate_object(
                        the_BookRepository_BorrowableCollection.in()
                    );
        objV = poaV->id_to_reference(oid);
        write_reference(objV,"BookRepository_BorrowableCollection.ref");
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("Borrowable");
        objectName[1].kind = CORBA::string_dup("");
        rootContextV->rebind(objectName, objV);

        // Create a servant for interface BookRepository::FlexibleCollection.
        the_BookRepository_FlexibleCollection =
            BookRepository_FlexibleCollectionImpl::_create(poaV);
        oid = poaV->activate_object(
                        the_BookRepository_FlexibleCollection.in()
                    );
        objV = poaV->id_to_reference(oid);
        write_reference(objV,"BookRepository_FlexibleCollection.ref");
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("BookRepository");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("Flexible");
        objectName[1].kind = CORBA::string_dup("");
        rootContextV->rebind(objectName, objV);

        // Activate the POA Manager and let the ORB process requests.
        root_poa_managerV->activate();
        cout << "Waiting for requests..." << endl;
        global_orb->run();
    }
    catch (CORBA::Exception& e)
    {
        cout << "Unexpected CORBA exception: " << e << endl;
    }

    // Servants are automatically deleted by '_var' types.

    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return 0;
}
