
package Pure.Util;

import org.omg.CORBA.*;
import org.omg.CosNaming.*; 
import org.omg.CosNaming.NamingContextPackage.*; 
import org.omg.PortableServer.POAManagerPackage.*; 

public class NameUtil {

  public static void printName(NameComponent[] name)
	  throws org.omg.CORBA.UserException
  {
     for (int i=0; i<name.length; i++) {
          if ( name[i].id.length()==0 ) {
	       //Special case: zero length 'id'
	       System.out.print("." + name[i].kind);
	  }
	  else {
	       //Regular case
	       System.out.print(name[i].id);
               if ( name[i].kind.length()!=0 ) {
	            System.out.print("." + name[i].kind);
	       }
	  }
	  if (i < name.length-1) {
	       System.out.print("/");
	  }
     }
  }

  //----------------------------------------
  // method: 'createContextPath()'
  //
  // purpose: For each 'NameComponent' in 'name',
  //          create a corresponding 'NamingContext'.
  //----------------------------------------
  public static void createContextPath(
          NamingContext nc,
          NameComponent[]  name
	  )
	  throws org.omg.CORBA.UserException
  {
    boolean isNotFound = false;
    int lengthMissing = 0;

    NamingContext tmpCtx;
    try {
        tmpCtx = nc.bind_new_context(name);
    }
    catch (NotFound nf) {
        isNotFound = true;
	lengthMissing = nf.rest_of_name.length;
    }

    if (isNotFound && lengthMissing==name.length ) {
        System.err.println("This cannot happen!");
	return;
    }

    if (isNotFound) {
        for (int len = name.length-lengthMissing;
	                  len <= name.length;
			  len++)
        {
            NameComponent[] tmpName = new NameComponent[len];
	    for (int i=0; i < len; i++) { tmpName[i] = name[i]; }
            tmpCtx = nc.bind_new_context(tmpName);
	}
    }

  }

  //----------------------------------------
  // method: 'bindObjectPath()'
  //
  // purpose: For the first [0, length-2] NameComponents of
  //          'name', create a corresponding 'NamingContext'.
  //          For the length-1 NameComponent of 'name', bind it
  //          to the object reference 'obj'.
  //----------------------------------------
  public static void bindObjectPath(
          NamingContext nc,
          NameComponent[]  name,
	  org.omg.CORBA.Object obj
	  )
	  throws org.omg.CORBA.UserException
  {
     try {
          nc.rebind(name, obj);
     }
     catch (NotFound nf) {
         NameComponent[] tmpName = new NameComponent[name.length-1];
	 for (int i=0; i < name.length-1; i++) { tmpName[i] = name[i]; }
         createContextPath(nc, tmpName);

         nc.bind(name, obj);
     }
  }

  //----------------------------------------
  // method: 'listBindings()'
  //
  // purpose: 
  //----------------------------------------
  public static Binding[] listBindings(
          NamingContext nc,
          NameComponent[]  name,
	  int max_list_size
	  )
	  throws org.omg.CORBA.UserException
  {
     Binding[]     basicList;
     BindingIterator bIter;

     org.omg.CORBA.Object obj;
     NamingContext tmpContext;

     if (name.length==0) {
          tmpContext = nc;
     }
     else {
          obj = nc.resolve(name);
          tmpContext = NamingContextHelper.narrow(obj);
     }

     BindingListHolder basicList_out
          = new BindingListHolder();
     BindingIteratorHolder bIter_out
          = new BindingIteratorHolder();
     tmpContext.list(
          max_list_size,
          basicList_out,
	  bIter_out
	  );
     basicList = basicList_out.value;
     bIter     = bIter_out.value;

     int      max_remaining = max_list_size - basicList.length;
     boolean  moreBindings  = (bIter!=null);

     if (moreBindings) {
	  while (moreBindings && (max_remaining > 0) ) {
               Binding[] tmpList;
               Binding[] oldList;
	       BindingListHolder tmpList_out
	            = new BindingListHolder();

	       moreBindings = bIter.next_n(
	            max_list_size,
		    tmpList_out
		    );
	       tmpList = tmpList_out.value;
               
	       //Append 'tmpList' to 'basicList'
	       oldList = basicList;
	       basicList = new Binding[oldList.length+tmpList.length];
	       for (int i=0; i < oldList.length; i++) {
	           basicList[i] = oldList[i];
	       }
	       for (int i=0; i < tmpList.length; i++) {
	            basicList[i+oldList.length] = tmpList[i];
	       }

	       // Re-calculate 'max_remaining'
               max_remaining = max_list_size - basicList.length;
	  }
	  bIter.destroy();
     }

     return basicList;
}

  //----------------------------------------
  // method: 'printBindings()'
  //
  // purpose: 
  //----------------------------------------
  public static void printBindings(
          NamingContext nc,
          NameComponent[]  name
	  )
	  throws org.omg.CORBA.UserException
  {
     Binding[] bl;

     bl = NameUtil.listBindings(nc,
                        name,
			10000   // max_list_size
			);

     for (int i=0; i < bl.length; i++)
     {
          NameUtil.printName( bl[i].binding_name );
	  System.out.print("\t");

	  if ( bl[i].binding_type==BindingType.nobject ) {
	      System.out.println("Object");
	  }
	  else if ( bl[i].binding_type==BindingType.ncontext ) {
	      System.out.println("Context");
	  }
	  else {
	      System.out.println("?????");
	  }
     }
  }

  //----------------------------------------
  // method: 'recursiveUnbind()'
  //
  // purpose: 
  //----------------------------------------
  public static void recursiveUnbind(
          NamingContext nc,
          NameComponent[]  name
	  )
	  throws org.omg.CORBA.UserException
  {
     org.omg.CORBA.Object obj;
     NamingContext tmpContext;

     obj = nc.resolve(name);
     tmpContext = NamingContextHelper.narrow(obj);

     Binding[] bl;
     NameComponent[] tmpName = new NameComponent[0];

     bl = NameUtil.listBindings(tmpContext,
                        tmpName,
			10000   // max_list_size
			);

     for (int i=0; i < bl.length; i++)
     {
	  tmpName = bl[i].binding_name;

	  if      ( bl[i].binding_type==BindingType.nobject)
	  {
	       tmpContext.unbind(tmpName);
	  }
	  else if ( bl[i].binding_type==BindingType.ncontext)
	  {
	       NameUtil.recursiveUnbind(tmpContext, tmpName);
	  }
     }
     nc.unbind(name);
     tmpContext.destroy();
  }
} // class NameUtil
