// Class: RecycleBroker_Buying_i
// A POA servant which implements of the RecycleBroker::Buying interface
//

#ifndef RECYCLEBROKER_BUYING_I_H_
#define RECYCLEBROKER_BUYING_I_H_

#include "RecycleBrokerS.hh"
#include "RecycleBroker_CustomerAdmin_i.h"
#include "RecycleBroker_WasteItemAdmin_i.h"

class RecycleBroker_Buying_i :
    public virtual PortableServer::RefCountServantBase,
    public virtual POA_RecycleBroker::Buying
{
public:
    RecycleBroker_Buying_i(RecycleBroker::KeyType branch_id);

    virtual ~RecycleBroker_Buying_i();

    // _create() -- create a new servant.
    // Hides the difference between direct inheritance and tie servants.
    //    
    static POA_RecycleBroker::Buying*
    _create(RecycleBroker::KeyType branch_id);


    // IDL operations
    //
    virtual RecycleBroker::Customer_ptr
    create_customer(
        const RecycleBroker::CustomerDetails& initialData,
        RecycleBroker::KeyType_out      customer_id
    ) IT_THROW_DECL((
        CORBA::SystemException
    ));

    virtual RecycleBroker::Customer_ptr
    log_on(
        const char*                     name,
        const char*                     password
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NoPermission
    ));

    virtual void
    buy_item(
        RecycleBroker::KeyType          wasteitem_id,
        CORBA::Long                     quantity
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotFound,
        RecycleBroker::InsufficientQuantity,
        RecycleBroker::NetworkError,
        RecycleBroker::NotLoggedOn
    ));

    virtual void
    log_off() IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotLoggedOn
    ));


    // IDL attributes
    //
    virtual RecycleBroker::Buying::Status current_status() IT_THROW_DECL((CORBA::SystemException));

    virtual PortableServer::POA_ptr
    _default_POA();

private: 
    // Private members
    RecycleBroker::Buying::Status      m_current_status;
    RecycleBroker::KeyType              m_branch_id;
    RecycleBroker::KeyType              m_customer_id;
    RecycleBroker_CustomerAdmin_i*      m_customer_admin_iP;
    RecycleBroker_WasteItemAdmin_i*     m_wasteitem_admin_iP;
    
    // Private functions
    void _init_customer_admin();
    void _init_wasteitem_admin();

    // The following are not implemented
    RecycleBroker_Buying_i(const RecycleBroker_Buying_i &);
    RecycleBroker_Buying_i& operator=(const RecycleBroker_Buying_i &);
}; 

#endif
