// Servant which implements the RecycleBroker::Customer interface.
//

#include <stdlib.h>
#include <iostream.h>
#include "RecycleBroker_Customer_i.h"
#include "POAMapper.h"
#include "DBBase.h"


// _create() -- create a new servant.
POA_RecycleBroker::Customer*
RecycleBroker_Customer_i::_create(
    RecycleBroker::KeyType customer_id
)
{
    return new RecycleBroker_Customer_i(customer_id);
}


// RecycleBroker_Customer_i constructor
RecycleBroker_Customer_i::RecycleBroker_Customer_i(
    RecycleBroker::KeyType customer_id
)
{
    // Initialize 'm_dataP' with 'CustomerDetailsFull'
    m_hasValidCache = 0;
    if (global_db) {
        if (global_db->find_customer_details(customer_id, m_dataP) ) {
            m_hasValidCache = 1;
        }
    }
    if (!m_hasValidCache) {
        cerr << "error: RecycleBroker_Customer_i() - "
             << "Failed to initialize cache with customer_id = "
             << customer_id << endl;
    }
}

RecycleBroker_Customer_i::RecycleBroker_Customer_i(
    const RecycleBroker::CustomerDetailsFull& data
)
{
    m_dataP = new RecycleBroker::CustomerDetailsFull();
    *m_dataP = data;
    m_hasValidCache = 1;
}

// ~RecycleBroker_Customer_i destructor.
//
RecycleBroker_Customer_i::~RecycleBroker_Customer_i()
{
    delete m_dataP;
}


// get_details() -- Implements IDL operation "RecycleBroker::Customer::get_details".
//
RecycleBroker::CustomerDetails*
RecycleBroker_Customer_i::get_details() IT_THROW_DECL((
    CORBA::SystemException
))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    RecycleBroker::CustomerDetails_var resultV
        = new RecycleBroker::CustomerDetails();
    *resultV = m_dataP->public_details;

    return resultV._retn();
}


// name() -- Accessor for IDL attribute "RecycleBroker::Customer::name".
//
char*
RecycleBroker_Customer_i::name() IT_THROW_DECL((CORBA::SystemException))

{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    return CORBA::string_dup( (m_dataP->public_details).name);
}


// name() -- Modifier for IDL attribute "RecycleBroker::Customer::name".
//
void
RecycleBroker_Customer_i::name(
    const char*                     _new_value
) IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    (m_dataP->public_details).name = CORBA::string_dup(_new_value);
    _update_db_from_cache();
}


// address() -- Accessor for IDL attribute "RecycleBroker::Customer::address".
//
char*
RecycleBroker_Customer_i::address() IT_THROW_DECL((CORBA::SystemException))

{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    return CORBA::string_dup( (m_dataP->public_details).address);
}


// address() -- Modifier for IDL attribute "RecycleBroker::Customer::address".
//
void
RecycleBroker_Customer_i::address(
    const char*                     _new_value
) IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    (m_dataP->public_details).address = CORBA::string_dup(_new_value);
    _update_db_from_cache();
}


// email_address() -- Accessor for IDL attribute "RecycleBroker::Customer::email_address".
//
char*
RecycleBroker_Customer_i::email_address() IT_THROW_DECL((CORBA::SystemException))

{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    return CORBA::string_dup( (m_dataP->public_details).email_address);
}


// email_address() -- Modifier for IDL attribute "RecycleBroker::Customer::email_address".
//
void
RecycleBroker_Customer_i::email_address(
    const char*                     _new_value
) IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    (m_dataP->public_details).email_address = CORBA::string_dup(_new_value);
    _update_db_from_cache();
}


// password() -- Accessor for IDL attribute "RecycleBroker::Customer::password".
//
char*
RecycleBroker_Customer_i::password() IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    return CORBA::string_dup( (m_dataP->public_details).password);
}


// password() -- Modifier for IDL attribute "RecycleBroker::Customer::password".
//
void
RecycleBroker_Customer_i::password(
    const char*                     _new_value
) IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    (m_dataP->public_details).password = CORBA::string_dup(_new_value);
    _update_db_from_cache();
}


// branch_id() -- Accessor for IDL attribute "RecycleBroker::Customer::branch_id".
//
RecycleBroker::KeyType
RecycleBroker_Customer_i::branch_id() IT_THROW_DECL((CORBA::SystemException))

{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    return m_dataP->branch_id;
}


// customer_id() -- Accessor for IDL attribute "RecycleBroker::Customer::customer_id".
//
RecycleBroker::KeyType
RecycleBroker_Customer_i::customer_id() IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    
    return m_dataP->customer_id;
}

PortableServer::POA_ptr
RecycleBroker_Customer_i::_default_POA()
{
    CORBA::String_var rep_idV = RecycleBroker::_tc_Customer->id();
    return POAMapper::getPOA(rep_idV.in() );
}

// Private functions
void
RecycleBroker_Customer_i::_update_db_from_cache()
{
    RecycleBroker::KeyType customer_id = m_dataP->customer_id;
    CORBA::Boolean success = 0;
    if (global_db) {
        if (global_db->update_customer_details(
                           customer_id,
                           m_dataP->public_details
                       ) )
        {
            success = 1;
        }
    }
    if (!success) {
        cerr << "error: RecycleBroker_Customer_i::_update_db_from_cache() - "
             << "Failed to update DB with customer_id = "
             << customer_id << endl;
    }
}

