// Class: RecycleBroker_Selling_i
// A POA servant which implements of the RecycleBroker::Selling interface
//

#ifndef RECYCLEBROKER_SELLING_I_H_
#define RECYCLEBROKER_SELLING_I_H_

#include "RecycleBrokerS.hh"
#include "RecycleBroker_CustomerAdmin_i.h"
#include "RecycleBroker_WasteItemAdmin_i.h"


class RecycleBroker_Selling_i :
    public virtual PortableServer::RefCountServantBase,
    public virtual POA_RecycleBroker::Selling
{
public:
    RecycleBroker_Selling_i(RecycleBroker::KeyType branch_id);

    virtual ~RecycleBroker_Selling_i();

    // _create() -- create a new servant.
    // Hides the difference between direct inheritance and tie servants.
    //    
    static POA_RecycleBroker::Selling*
    _create(RecycleBroker::KeyType branch_id);


    // IDL operations
    //
    virtual RecycleBroker::Customer_ptr
    create_customer(
        const RecycleBroker::CustomerDetails& initialData,
        RecycleBroker::KeyType_out      customer_id
    ) IT_THROW_DECL((
        CORBA::SystemException
    ));

    virtual RecycleBroker::Customer_ptr
    log_on(
        const char*                     name,
        const char*                     password
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NoPermission
    ));

    virtual RecycleBroker::WasteItem_ptr
    create_waste_item(
        const RecycleBroker::WasteItemDetails& initialData,
        RecycleBroker::KeyType_out      wasteitem_id
    ) IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotLoggedOn
    ));

    virtual void
    log_off() IT_THROW_DECL((
        CORBA::SystemException,
        RecycleBroker::NotLoggedOn
    ));


    // IDL attributes
    //
    virtual RecycleBroker::Selling::Status current_status() IT_THROW_DECL((CORBA::SystemException));

    virtual PortableServer::POA_ptr
    _default_POA();

private: 
    // Private members
    RecycleBroker::Selling::Status      m_current_status;
    RecycleBroker::KeyType              m_branch_id;
    RecycleBroker::KeyType              m_customer_id;
    RecycleBroker_CustomerAdmin_i*      m_customer_admin_iP;
    RecycleBroker_WasteItemAdmin_i*     m_wasteitem_admin_iP;
    
    // Private functions
    void _init_customer_admin();
    void _init_wasteitem_admin();

    // The following are not implemented
    RecycleBroker_Selling_i(const RecycleBroker_Selling_i &);
    RecycleBroker_Selling_i& operator=(const RecycleBroker_Selling_i &);
}; 

#endif
