// Servant which implements the RecycleBroker::WasteItem interface.
//

#include <stdlib.h>
#include <iostream.h>
#include "RecycleBroker_WasteItem_i.h"
#include "POAMapper.h"
#include "DBBase.h"


// _create() -- create a new servant.
POA_RecycleBroker::WasteItem*
RecycleBroker_WasteItem_i::_create(RecycleBroker::KeyType wasteitem_id)
{
    return new RecycleBroker_WasteItem_i(wasteitem_id);
}


// RecycleBroker_WasteItem_i constructor
RecycleBroker_WasteItem_i::RecycleBroker_WasteItem_i(
    RecycleBroker::KeyType wasteitem_id
)
{
    // Initialize 'm_dataP' with 'WasteItemDetailsFull'
    m_hasValidCache = 0;
    if (global_db) {
        if (global_db->find_wasteitem_details(wasteitem_id, m_dataP) ) {
            m_hasValidCache = 1;
        }
    }
    if (!m_hasValidCache) {
        cerr << "error: RecycleBroker_WasteItem_i() - "
             << "Failed to initialize cache with wasteitem_id = "
             << wasteitem_id << endl;
    }
}

RecycleBroker_WasteItem_i::RecycleBroker_WasteItem_i(
    const RecycleBroker::WasteItemDetailsFull& data
)
{
    m_dataP = new RecycleBroker::WasteItemDetailsFull();
    *m_dataP = data;
    m_hasValidCache = 1;
}


// ~RecycleBroker_WasteItem_i destructor.
//
RecycleBroker_WasteItem_i::~RecycleBroker_WasteItem_i()
{
    delete m_dataP;
}


// get_details() -- Implements IDL operation "RecycleBroker::WasteItem::get_details".
//
RecycleBroker::WasteItemDetails
RecycleBroker_WasteItem_i::get_details() IT_THROW_DECL((
    CORBA::SystemException
))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    return m_dataP->public_details;
}


// waste() -- Accessor for IDL attribute "RecycleBroker::WasteItem::waste".
//
RecycleBroker::WasteType
RecycleBroker_WasteItem_i::waste() IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    return (m_dataP->public_details).waste;
}


// waste() -- Modifier for IDL attribute "RecycleBroker::WasteItem::waste".
//
void
RecycleBroker_WasteItem_i::waste(
    RecycleBroker::WasteType        _new_value
) IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    (m_dataP->public_details).waste = _new_value;
    _update_db_from_cache();
}


// quantity() -- Accessor for IDL attribute "RecycleBroker::WasteItem::quantity".
//
CORBA::Long
RecycleBroker_WasteItem_i::quantity() IT_THROW_DECL((CORBA::SystemException))

{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    return (m_dataP->public_details).quantity;
}


// quantity() -- Modifier for IDL attribute "RecycleBroker::WasteItem::quantity".
//
void
RecycleBroker_WasteItem_i::quantity(
    CORBA::Long                     _new_value
) IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    (m_dataP->public_details).quantity = _new_value;
    _update_db_from_cache();
}


// price_per_kilo() -- Accessor for IDL attribute "RecycleBroker::WasteItem::price_per_kilo".
//
RecycleBroker::PriceType
RecycleBroker_WasteItem_i::price_per_kilo() IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    return (m_dataP->public_details).price_per_kilo;
}


// price_per_kilo() -- Modifier for IDL attribute "RecycleBroker::WasteItem::price_per_kilo".
//
void
RecycleBroker_WasteItem_i::price_per_kilo(
    RecycleBroker::PriceType        _new_value
) IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    (m_dataP->public_details).price_per_kilo = _new_value;
    _update_db_from_cache();
}


// branch_id() -- Accessor for IDL attribute "RecycleBroker::WasteItem::branch_id".
//
RecycleBroker::KeyType
RecycleBroker_WasteItem_i::branch_id() IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    return m_dataP->branch_id;
}


// customer_id() -- Accessor for IDL attribute "RecycleBroker::WasteItem::customer_id".
//
RecycleBroker::KeyType
RecycleBroker_WasteItem_i::customer_id() IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    return m_dataP->customer_id;
}


// wasteitem_id() -- Accessor for IDL attribute "RecycleBroker::WasteItem::wasteitem_id".
//
RecycleBroker::KeyType
RecycleBroker_WasteItem_i::wasteitem_id() IT_THROW_DECL((CORBA::SystemException))
{
    if (!m_hasValidCache) { throw CORBA::PERSIST_STORE(); }
    return m_dataP->wasteitem_id;
}

PortableServer::POA_ptr
RecycleBroker_WasteItem_i::_default_POA()
{
    CORBA::String_var rep_idV = RecycleBroker::_tc_WasteItem->id();
    return POAMapper::getPOA(rep_idV.in() );
}

// Private functions
void
RecycleBroker_WasteItem_i::_update_db_from_cache()
{
    RecycleBroker::KeyType wasteitem_id = m_dataP->wasteitem_id;
    CORBA::Boolean success = 0;
    if (global_db) {
        if (global_db->update_wasteitem_details(
                           wasteitem_id,
                           m_dataP->public_details
                       ) )
        {
            success = 1;
        }
    }
    if (!success) {
        cerr << "error: RecycleBroker_WasteItem_i::_update_db_from_cache() - "
             << "Failed to update DB with wasteitem_id = "
             << wasteitem_id << endl;
    }
}

