#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string>
#include "RecycleBroker.hh"


// global_orb -- make ORB global so all code can find it.
CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();


// read_reference() -- read an object reference from file.
// Reads a stringified object reference from file and returns a real
// object reference.
// Caller must release the returned reference or assign it to a _var. 
// On error returns a nil object reference.
//
static 
CORBA::Object_ptr
read_reference(
    const char*      file
)
{
    cout << "Reading stringified object reference from " << file << endl;
    ifstream ifs(file);
    CORBA::String_var str;
    ifs >> str;
    if (!ifs) {
        cerr << "Error reading object reference from " << file << endl;
        return CORBA::Object::_nil();
    }
    return global_orb->string_to_object(str);
}

static void
print_wastetype_menu()
{
    cout << "\t0\tBROWN_GLASS" << endl;
    cout << "\t1\tGREEN_GLASS" << endl;
    cout << "\t2\tCLEAR_GLASS" << endl;
    cout << "\t3\tSCRAP_STEEL" << endl;
    cout << "\t4\tALUMINIUM_CANS" << endl;
    cout << "\t5\tPLASTIC_BOTTLES" << endl;
    cout << "\t6\tWASTE_PAPER" << endl;
}

static void
print_wasteitem_details(
    const RecycleBroker::WasteItemDetails& details,
    const char* indent
)
{
    cout << indent << "waste:\t\t";
    switch (details.waste) {
        case RecycleBroker::BROWN_GLASS:
            cout << "BROWN_GLASS" << endl;
            break;
        case RecycleBroker::GREEN_GLASS:
            cout << "GREEN_GLASS" << endl;
            break;
        case RecycleBroker::CLEAR_GLASS:
            cout << "CLEAR_GLASS" << endl;
            break;
        case RecycleBroker::SCRAP_STEEL:
            cout << "SCRAP_STEEL" << endl;
            break;
        case RecycleBroker::ALUMINIUM_CANS:
            cout << "ALUMINIUM_CANS" << endl;
            break;
        case RecycleBroker::PLASTIC_BOTTLES:
            cout << "PLASTIC_BOTTLES" << endl;
            break;
        case RecycleBroker::WASTE_PAPER:
            cout << "WASTE_PAPER" << endl;
            break;
        default:
            cout << "<unknown>" << endl;
            break;
    }

    cout << indent << "quantity:\t" << details.quantity << endl;

    cout << indent << "price_per_kilo:\t" << details.price_per_kilo << endl;
}


int
main(int argc, char **argv)
{
    int exit_status = 0;        
    try
    {
        CORBA::Object_var tmp_ref;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

        // Get reference to 'RecycleBroker::BranchOffice'.
        tmp_ref = read_reference("RecycleBroker_BranchOffice.ref");
        RecycleBroker::BranchOffice_var branchOfficeV =
            RecycleBroker::BranchOffice::_narrow(tmp_ref);
        if (CORBA::is_nil(branchOfficeV.in() )) 
        {
            cerr << "Could not narrow reference to interface "
                 << "RecycleBroker::BranchOffice" << endl;
            exit(1);
        }

        // Use 'RecycleBroker::Selling' interface.
        tmp_ref = branchOfficeV->get_selling();
        RecycleBroker::Selling_var sellingV =
            RecycleBroker::Selling::_narrow(tmp_ref);
        if (CORBA::is_nil(sellingV.in() )) 
        {
            cerr << "Could not narrow reference to interface "
                 << "RecycleBroker::Selling" << endl;
            exit(1);
        }

        // 1. Log on
        cout << "Log on" << endl;
        char s[256];
        cout << "\tname:\t";
        cin.getline(s, 256);
        CORBA::String_var logon_nameV = CORBA::string_dup(s);
        cout << "\tpasswd:\t";
        cin.getline(s, 256);
        CORBA::String_var logon_passwdV = CORBA::string_dup(s);
        
        sellingV->log_on(logon_nameV.in(), logon_passwdV.in() );
        

        // 2. Post a waste item for sale.
        cout << endl << "Offer waste item for sale:" << endl;
        RecycleBroker::WasteItemDetails data;
        RecycleBroker::KeyType          wasteitem_id;
        print_wastetype_menu();
        cout << endl << "enter waste type by number:\t";
        CORBA::Long waste_num;
        cin >> waste_num;
        data.waste = (RecycleBroker::WasteType) waste_num;
        cout << "enter quantity (kilogrammes):\t";
        cin >> data.quantity;
        cout << "enter price per kilo:\t\t";
        cin >> data.price_per_kilo;
        RecycleBroker::WasteItem_var wasteitemV;
        wasteitemV = sellingV->create_waste_item(data, wasteitem_id);
        cout << endl
             << "Created waste item with wasteitem_id = "
             << wasteitem_id << endl;
        try {
            cout << endl << "Checking details:" << endl;
            RecycleBroker::WasteItemDetails_var detailsV;
            detailsV = wasteitemV->get_details();
            print_wasteitem_details(detailsV.in(), "\t");
        }
        catch (RecycleBroker::NotFound& ) {
            cerr << "error: waste item not found." << endl;
        }

        // 3. Log off
        sellingV->log_off();
        cout << endl << "Logged off" << endl;
    }
    catch (RecycleBroker::NoPermission& )
    {
        cerr << "error: Logon failed." << endl;
    }
    catch (CORBA::Exception &ex) 
    {
        cerr << "error: Unexpected CORBA exception: " << ex << endl;
        exit_status = 1;
    }

    // Ensure that the ORB is properly shutdown and cleaned up.
    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return exit_status;
}


